@@ -1,3 +1,21 @@
+/* AbiSource Program Utilities
+ * Copyright (C) 2001 Tomas Frydrych
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
+ */
 
 #include "ut_Language.h"
 #include "ut_string.h"
@@ -21,9 +39,15 @@ static lang_entry s_Table[] =
 	{"af-ZA",		NULL, XAP_STRING_ID_LANG_AF_ZA},
 	{"ar-EG",		NULL, XAP_STRING_ID_LANG_AR_EG},
 	{"ar-SA",		NULL, XAP_STRING_ID_LANG_AR_SA},
+	{"as-IN",		NULL, XAP_STRING_ID_LANG_AS_IN},
+	{"be-BY",		NULL, XAP_STRING_ID_LANG_BE_BY},
+	{"bg-BG",		NULL, XAP_STRING_ID_LANG_BG_BG},
+	{"ca-ES",		NULL, XAP_STRING_ID_LANG_CA_ES},
 	{"cs-CZ",		NULL, XAP_STRING_ID_LANG_CS_CZ},
+	{"cy-GB",		NULL, XAP_STRING_ID_LANG_CY_GB},
 	{"da-DK",		NULL, XAP_STRING_ID_LANG_DA_DK},
 	{"de-AT",		NULL, XAP_STRING_ID_LANG_DE_AT},
+	{"de-CH",		NULL, XAP_STRING_ID_LANG_DE_CH},
 	{"de-DE",		NULL, XAP_STRING_ID_LANG_DE_DE},
 	{"el-GR",		NULL, XAP_STRING_ID_LANG_EL_GR},
 	{"en-AU",		NULL, XAP_STRING_ID_LANG_EN_AU},
@@ -33,34 +57,53 @@ static lang_entry s_Table[] =
 	{"en-NZ",		NULL, XAP_STRING_ID_LANG_EN_NZ},
 	{"en-US",		NULL, XAP_STRING_ID_LANG_EN_US},
 	{"en-ZA",		NULL, XAP_STRING_ID_LANG_EN_ZA},
+	{"eo",			NULL, XAP_STRING_ID_LANG_EO},
 	{"es-ES",		NULL, XAP_STRING_ID_LANG_ES_ES},
 	{"es-MX",		NULL, XAP_STRING_ID_LANG_ES_MX},
+	{"eu-ES",		NULL, XAP_STRING_ID_LANG_EU_ES},
 	{"fa-IR",		NULL, XAP_STRING_ID_LANG_FA_IR},
 	{"fi-FI",		NULL, XAP_STRING_ID_LANG_FI_FI},
+	{"fr-BE",		NULL, XAP_STRING_ID_LANG_FR_BE},
+	{"fr-CA",		NULL, XAP_STRING_ID_LANG_FR_CA},
+	{"fr-CH",		NULL, XAP_STRING_ID_LANG_FR_CH},
 	{"fr-FR",		NULL, XAP_STRING_ID_LANG_FR_FR},
+	{"ga-IE",		NULL, XAP_STRING_ID_LANG_GA_IE},
+	{"gl-ES",		NULL, XAP_STRING_ID_LANG_GL_ES},
+	{"he-IL",		NULL, XAP_STRING_ID_LANG_HE_IL},	// was iw-IL - why?
 	{"hi-IN",		NULL, XAP_STRING_ID_LANG_HI_IN},
+	{"hu-HU",		NULL, XAP_STRING_ID_LANG_HU_HU},
 	{"hy-AM",		NULL, XAP_STRING_ID_LANG_HY_AM},	// Win2K shows "hy-am"
+	{"is-IS",		NULL, XAP_STRING_ID_LANG_IS_IS},
+	{"ia",			NULL, XAP_STRING_ID_LANG_IA},
+	{"id-ID",		NULL, XAP_STRING_ID_LANG_ID_ID},
 	{"it-IT",		NULL, XAP_STRING_ID_LANG_IT_IT},
-	{"iw-IL",		NULL, XAP_STRING_ID_LANG_IW_IL},
 	{"ja-JP",		NULL, XAP_STRING_ID_LANG_JA_JP},
 	{"ka-GE",		NULL, XAP_STRING_ID_LANG_KA_GE},
 	{"ko-KR",		NULL, XAP_STRING_ID_LANG_KO_KR},
+	{"la-IT",		NULL, XAP_STRING_ID_LANG_LA_IT},
 	{"lt-LT",		NULL, XAP_STRING_ID_LANG_LT_LT},
 	{"lv-LV",		NULL, XAP_STRING_ID_LANG_LV_LV},
 	{"nb-NO",		NULL, XAP_STRING_ID_LANG_NB_NO},
+	{"nl-BE",		NULL, XAP_STRING_ID_LANG_NL_BE},
 	{"nl-NL",		NULL, XAP_STRING_ID_LANG_NL_NL},
 	{"nn-NO",		NULL, XAP_STRING_ID_LANG_NN_NO},
+	{"pl-PL",		NULL, XAP_STRING_ID_LANG_PL_PL},
 	{"pt-BR",		NULL, XAP_STRING_ID_LANG_PT_BR},
 	{"pt-PT",		NULL, XAP_STRING_ID_LANG_PT_PT},
+	{"ro-RO",		NULL, XAP_STRING_ID_LANG_RO_RO},
 	{"ru-RU",		NULL, XAP_STRING_ID_LANG_RU_RU},
+	{"sk-SK",		NULL, XAP_STRING_ID_LANG_SK_SK},
+	{"sl-SI",		NULL, XAP_STRING_ID_LANG_SL_SI},
 	{"sv-SE",		NULL, XAP_STRING_ID_LANG_SV_SE},
 	{"th-TH",		NULL, XAP_STRING_ID_LANG_TH_TH},
 	{"tr-TR",		NULL, XAP_STRING_ID_LANG_TR_TR},
+	{"uk-UA",		NULL, XAP_STRING_ID_LANG_UK_UA},
 	{"vi-VN",		NULL, XAP_STRING_ID_LANG_VI_VN},
 	{"zh-CN",		NULL, XAP_STRING_ID_LANG_ZH_CN},
 	{"zh-HK",		NULL, XAP_STRING_ID_LANG_ZH_HK},
 	{"zh-SG",		NULL, XAP_STRING_ID_LANG_ZH_SG},
 	{"zh-TW",		NULL, XAP_STRING_ID_LANG_ZH_TW},
+	{"yi",			NULL, XAP_STRING_ID_LANG_YI},
 };
 
 static int s_compareQ(const void * a, const void *b)
