@@ -39,9 +39,9 @@
 
 #ifdef _WIN32
 #define finite _finite
-double rint(double x);
+ABI_EXPORT double rint(double x);
 #endif /* _WIN32 */
 
-UT_uint32 UT_newNumber ();
+ABI_EXPORT UT_uint32 UT_newNumber ();
 
 #endif /* UTMATH_H */
