@@ -543,6 +543,13 @@ void UT_parseColor(const char *p, UT_RGBColor& c)
 	    return ;
 	  }
 
+	if(!strcmp(p,"transparent") /* || !strcmp(p,"ffffff") */)
+	{
+		c.m_red = c.m_grn = c.m_blu = 255;
+		//c.m_bIsTransparent = true;
+		return;
+	}
+
 	UT_HashColor hash;
 
 	if (hash.setColor (p))
@@ -663,7 +670,7 @@ bool UT_isWordDelimiter(UT_UCSChar currentChar, UT_UCSChar followChar, UT_UCSCha
 				  return true;
 			  }
 		case UCS_ABI_OBJECT:
-			return false;
+			return true;
 
 		default:
 			return s_find_delim(currentChar);
