@@ -37,11 +37,18 @@
 #include "ev_EditMethod.h"
 #include "xav_View.h"
 #include "xad_Document.h"
+#include "ie_impGraphic.h"
+#include "fv_View.h"
 
 #ifdef _MSC_VER
 #pragma warning(disable: 4355)	// 'this' used in base member initializer list
 #endif
 
+// TODO Fix the following header file. It seems to be incomplete
+// TODO #include <ap_EditMethods.h>
+// TODO In the mean time, define the needed function by hand
+extern XAP_Dialog_MessageBox::tAnswer s_CouldNotLoadFileMessage(XAP_Frame * pFrame, const char * pNewFile, UT_Error errorCode);
+
 /*****************************************************************/
 
 bool XAP_Win32Frame::RegisterClass(XAP_Win32App* app)
@@ -160,6 +167,7 @@ bool XAP_Win32Frame::initialize(const char* szKeyBindingsKey,
 	// TODO: Jeff, I'm currently showing in WinMain, to honor iCmdShow.
 	// should we pass that via argv, to do it here for all frames?
 
+
 	return true;
 }
 
@@ -199,15 +207,40 @@ void XAP_Win32Frame::_createTopLevelWindow(void)
 {
 	RECT r;
 	UT_uint32 iHeight, iWidth;
+	UT_sint32 iPosX, iPosY;
+	static bool firstWindow = true;	/* position only 1st window! */
 
 	// create a top-level window for us.
-	// TODO get the default window size from preferences or something.
-	// Handled in Win32App for 1st frame opened.
+	// get the default window size from preferences or something.
+	// should set size for all, but position only on 1st created
+	// TODO determine where to save & restore from Window flag (since
+	//      we can't use the geometry flag (its some other junk about validity of pos & size)
+	//      so we can properly restore Maximized/Minimized/Normal mode windows
+	// For now we can let Win32App set the flag for the 1st opened window
+
+	// get window width & height from preferences
+	UT_uint32 t_flag;		// dummy variable
+	if ( !(XAP_App::getApp()->getGeometry(&iPosX,&iPosY,&iWidth,&iHeight,&t_flag)) ||
+           !((iWidth > 0) && (iHeight > 0)) )
+	{
+		UT_DEBUGMSG(("Unable to obtain saved geometry, using window defaults!\n"));
+		iWidth = CW_USEDEFAULT;
+		iHeight = CW_USEDEFAULT;
+		iPosX = CW_USEDEFAULT;
+		iPosY = CW_USEDEFAULT;
+	}
+	/* let Windows(R) place the Window for all but 1st one, for stairstep effect */
+	if (!firstWindow)
+	{
+		iPosX = CW_USEDEFAULT;
+		iPosY = CW_USEDEFAULT;
+	}
+	else firstWindow = false;
 
 	m_hwndFrame = CreateWindow(m_pWin32App->getApplicationName(),
 							   m_pWin32App->getApplicationTitleForTitleBar(),
 							   WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN,
-							   CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
+							   iPosX, iPosY, iWidth, iHeight,
 							   NULL, NULL, m_pWin32App->getInstance(), NULL);
 	UT_ASSERT(m_hwndFrame);
 
@@ -282,6 +315,9 @@ void XAP_Win32Frame::_createTopLevelWindow(void)
 	GetClientRect(m_hwndStatusBar,&r);
 	m_iStatusBarHeight = r.bottom;
 
+	// Allow drag-and drop
+	DragAcceptFiles(m_hwndFrame, true);
+
 	// we let our caller decide when to show m_hwndFrame.
 
 	return;
@@ -299,7 +335,11 @@ bool XAP_Win32Frame::close()
 				wndPlacement.rcNormalPosition.top, 
 				wndPlacement.rcNormalPosition.right - wndPlacement.rcNormalPosition.left,
 				wndPlacement.rcNormalPosition.bottom - wndPlacement.rcNormalPosition.top,
+				/* flag is meant for info about the position & size info stored, not a generic flag
+				 * TODO: figure out where to store this then, so we can max/min/normal again
 				wndPlacement.showCmd
+				*/
+				PREF_FLAG_GEOMETRY_POS | PREF_FLAG_GEOMETRY_SIZE
 				);
 	}
 	else
@@ -334,11 +374,66 @@ bool XAP_Win32Frame::openURL(const char * szURL)
 	// NOTE: could get finer control over browser window via DDE 
 	// NOTE: may need to fallback to WinExec for old NSCP versions
 
+	UT_String sURL = szURL;
 	HWND hwnd = getTopLevelWindow();
-	int res = (int) ShellExecute(hwnd, "open", szURL, NULL, NULL, SW_SHOWNORMAL);
 
-	// TODO: more specific error messages ??
-	UT_ASSERT(res>32);
+	// strip "file://" from URL, win32 doesn't handle them well
+	if ( "file://" == sURL.substr(0, 7) )
+	{
+		sURL = sURL.substr(7, sURL.size() - 7);
+	}
+
+	int res = (int) ShellExecute(hwnd, "open", sURL.c_str(), NULL, NULL, SW_SHOWNORMAL);
+
+	// TODO: more specific (and localized) error messages ??
+	// added more specific error messages as documented in http://msdn.microsoft.com/library/default.asp?url=/library/en-us/debug/base/system_error_codes.asp
+
+	if (res <= 32)	// show error message if failed to launch browser to display URL
+	{
+		UT_String errMsg;
+		switch (res)
+		{
+			case 2:
+				{
+					errMsg = "Error ("; 
+					errMsg += UT_String_sprintf("%d", res);
+					errMsg += ") displaying URL: The system cannot find the file specified.\n";
+					errMsg += " [ ";  errMsg += sURL;  errMsg += " ] ";
+					MessageBox(hwnd, errMsg.c_str(), "Error displaying URL", MB_OK|MB_ICONEXCLAMATION);
+				}
+				break;
+
+			case 3:
+				{
+					errMsg = "Error ("; 
+					errMsg += UT_String_sprintf("%d", res);
+					errMsg += ") displaying URL: The system cannot find the path specified.\n";
+					errMsg += " [ ";  errMsg += sURL;  errMsg += " ] ";
+					MessageBox(hwnd, errMsg.c_str(), "Error displaying URL", MB_OK|MB_ICONEXCLAMATION);
+				}
+				break;
+
+			case 5:
+				{
+					errMsg = "Error ("; 
+					errMsg += UT_String_sprintf("%d", res);
+					errMsg += ") displaying URL: Access is denied.\n";
+					errMsg += " [ ";  errMsg += sURL;  errMsg += " ] ";
+					MessageBox(hwnd, errMsg.c_str(), "Error displaying URL", MB_OK|MB_ICONEXCLAMATION);
+				}
+				break;
+
+			default:
+				{
+					errMsg = "Error ("; 
+					errMsg += UT_String_sprintf("%d", res);
+					errMsg += ") displaying URL: \n";
+					errMsg += " [ ";  errMsg += sURL;  errMsg += " ] ";
+					MessageBox(hwnd, errMsg.c_str(), "Error displaying URL", MB_OK|MB_ICONEXCLAMATION);
+				}
+				break;
+		} /* switch (res) */
+	} /* if (res <= 32) */
 
 	return (res>32);
 }
@@ -538,101 +633,195 @@ LRESULT CALLBACK XAP_Win32Frame::_FrameWndProc(HWND hwnd, UINT iMsg, WPARAM wPar
 		// Process other notifications here
 		default:
 			break;
-		} 
+		} /* switch (((LPNMHDR) lParam)->code) */
 		break;
 
 	case WM_SIZE:
-	{
-		int nWidth = LOWORD(lParam);
-		int nHeight = HIWORD(lParam);
-
-		if (nWidth != (int) f->m_iSizeWidth && f->m_hwndRebar != NULL)
 		{
-			MoveWindow(f->m_hwndRebar, 0, 0, nWidth, f->m_iBarHeight, TRUE); 
-		}
+			int nWidth = LOWORD(lParam);
+			int nHeight = HIWORD(lParam);
 
-		// leave room for the toolbars and the status bar
-		nHeight -= f->m_iBarHeight;
-		nHeight -= f->m_iStatusBarHeight;
+			if( pView && !pView->isLayoutFilling() )
+			{
+				f->_startViewAutoUpdater();
 
-		if (f->m_hwndContainer)
-			MoveWindow(f->m_hwndContainer, 0, f->m_iBarHeight, nWidth, nHeight, TRUE);
+				if (nWidth != (int) f->m_iSizeWidth && f->m_hwndRebar != NULL)
+				{
+					MoveWindow(f->m_hwndRebar, 0, 0, nWidth, f->m_iBarHeight, TRUE); 
+				}
 
-		if (f->m_hwndStatusBar)
-			MoveWindow(f->m_hwndStatusBar, 0, f->m_iBarHeight+nHeight, nWidth, f->m_iStatusBarHeight, TRUE);
-			
-		f->m_iSizeWidth = nWidth;
-		f->m_iSizeHeight = nHeight;
+				// leave room for the toolbars and the status bar
+				nHeight -= f->m_iBarHeight;
+				nHeight -= f->m_iStatusBarHeight;
 
-		// If Dynamic Zoom recalculate zoom setting
-		f->updateZoom();
+				if (f->m_hwndContainer)
+					MoveWindow(f->m_hwndContainer, 0, f->m_iBarHeight, nWidth, nHeight, TRUE);
 
-		return 0;
-	}
+				if (f->m_hwndStatusBar)
+					MoveWindow(f->m_hwndStatusBar, 0, f->m_iBarHeight+nHeight, nWidth, f->m_iStatusBarHeight, TRUE);
+				
+				f->m_iSizeWidth = nWidth;
+				f->m_iSizeHeight = nHeight;
 
-	case WM_CLOSE:
-	{
-		XAP_App * pApp = f->getApp();
-		UT_ASSERT(pApp);
+				f->updateZoom();
+			}
 
-		const EV_EditMethodContainer * pEMC = pApp->getEditMethodContainer();
-		UT_ASSERT(pEMC);
-
-		EV_EditMethod * pEM = pEMC->findEditMethodByName("closeWindowX");
-		UT_ASSERT(pEM);						// make sure it's bound to something
-
-		if (pEM)
-		{
-			(*pEM->getFn())(pView,NULL);
 			return 0;
 		}
 
-		// let the window be destroyed
-		break;
-	}
-
-	case WM_INPUTLANGCHANGE:
-	{
-		UT_DEBUGMSG(("Frame received input language change\n"));
-
-		// This will remap the static tables used by all frames.
-		// (see the comment in ev_Win32Keyboard.cpp.)
-		ev_Win32Keyboard *pWin32Keyboard = static_cast<ev_Win32Keyboard *>(f->m_pKeyboard);
-		pWin32Keyboard->remapKeyboard((HKL)lParam);
-
-		// Do not propagate this message.
-		
-		return 1; //DefWindowProc(hwnd, iMsg, wParam, lParam);
-	}
-
-	case WM_MOUSEWHEEL:
-	{
-		return SendMessage(f->m_hwndContainer, iMsg, wParam, lParam);
-	}
-
-	case WM_SYSCOLORCHANGE:
-	{
-		if (f->m_hwndRebar)
+	case WM_CLOSE:
 		{
-			SendMessage(f->m_hwndRebar,WM_SYSCOLORCHANGE,0,0);
+			XAP_App * pApp = f->getApp();
+			UT_ASSERT(pApp);
 
-			REBARBANDINFO rbbi = { 0 };
-			rbbi.cbSize = sizeof(REBARBANDINFO);
-			rbbi.fMask = RBBIM_COLORS;
-			rbbi.clrFore = GetSysColor(COLOR_BTNTEXT);
-			rbbi.clrBack = GetSysColor(COLOR_BTNFACE);
+			const EV_EditMethodContainer * pEMC = pApp->getEditMethodContainer();
+			UT_ASSERT(pEMC);
 
-			UT_uint32 nrToolbars = f->m_vecToolbars.getItemCount();
-			for (UT_uint32 k=0; k < nrToolbars; k++)
-				SendMessage(f->m_hwndRebar, RB_SETBANDINFO,k,(LPARAM)&rbbi);
+			EV_EditMethod * pEM = pEMC->findEditMethodByName("closeWindowX");
+			UT_ASSERT(pEM);						// make sure it's bound to something
+
+			if (pEM)
+			{
+				(*pEM->getFn())(pView,NULL);
+				return 0;
+			}
+
+			// let the window be destroyed
+			break;
 		}
 
-		if (f->m_hwndContainer)
-			SendMessage(f->m_hwndContainer,WM_SYSCOLORCHANGE,0,0);
-		if (f->m_hwndStatusBar)
-			SendMessage(f->m_hwndStatusBar,WM_SYSCOLORCHANGE,0,0);
+	case WM_INPUTLANGCHANGE:
+		{
+			UT_DEBUGMSG(("Frame received input language change\n"));
+
+			// This will remap the static tables used by all frames.
+			// (see the comment in ev_Win32Keyboard.cpp.)
+			ev_Win32Keyboard *pWin32Keyboard = static_cast<ev_Win32Keyboard *>(f->m_pKeyboard);
+			pWin32Keyboard->remapKeyboard((HKL)lParam);
+
+			// Do not propagate this message.
+			
+			return 1; //DefWindowProc(hwnd, iMsg, wParam, lParam);
+		}
+
+	case WM_MOUSEWHEEL:
+		{
+			return SendMessage(f->m_hwndContainer, iMsg, wParam, lParam);
+		}
+
+	case WM_SYSCOLORCHANGE:
+		{
+			if (f->m_hwndRebar)
+			{
+				SendMessage(f->m_hwndRebar,WM_SYSCOLORCHANGE,0,0);
+
+				REBARBANDINFO rbbi = { 0 };
+				rbbi.cbSize = sizeof(REBARBANDINFO);
+				rbbi.fMask = RBBIM_COLORS;
+				rbbi.clrFore = GetSysColor(COLOR_BTNTEXT);
+				rbbi.clrBack = GetSysColor(COLOR_BTNFACE);
+
+				UT_uint32 nrToolbars = f->m_vecToolbars.getItemCount();
+				for (UT_uint32 k=0; k < nrToolbars; k++)
+					SendMessage(f->m_hwndRebar, RB_SETBANDINFO,k,(LPARAM)&rbbi);
+			}
+
+			if (f->m_hwndContainer)
+				SendMessage(f->m_hwndContainer,WM_SYSCOLORCHANGE,0,0);
+			if (f->m_hwndStatusBar)
+				SendMessage(f->m_hwndStatusBar,WM_SYSCOLORCHANGE,0,0);
+			return 0;
+		}
+
+	case WM_DROPFILES:
+		{
+			HDROP hDrop = (HDROP) wParam; 
+			// How many files were dropped?
+			int count = DragQueryFile(hDrop, 0xFFFFFFFF, NULL, 0);
+			char szFileName[_MAX_PATH];
+			int i,pathlength;
+			for (i=0; i<count; i++)
+			{
+				pathlength = DragQueryFile(hDrop, i, NULL, 0);
+				if (pathlength < _MAX_PATH)
+				{
+					DragQueryFile(hDrop, i, szFileName, _MAX_PATH);
+					XAP_App * pApp = f->getApp();
+					UT_ASSERT(pApp);
+					FV_View* pView = (FV_View *) f->getCurrentView();
+					XAP_Frame * pNewFrame = 0;
+					IEGraphicFileType iegft = IEGFT_Unknown;					
+					IE_ImpGraphic *pIEG;
+					FG_Graphic* pFG;
+					UT_Error errorCode;
+
+					/*
+						The user may be dropping every kind of file
+						Check first if the file is a graphic. If it's a graphics. I we insert it 
+						in the document, if not we assume that it's document 		
+					*/								
+					// If there is no import graphic, it's a document...
+					errorCode = IE_ImpGraphic::constructImporter(szFileName, iegft, &pIEG);
+					if(errorCode == UT_OK)
+					{						
+						errorCode = pIEG->importGraphic(szFileName, &pFG);
+						if(errorCode != UT_OK || !pFG)
+						{
+							s_CouldNotLoadFileMessage(f, szFileName, errorCode);							
+							DELETEP(pIEG);
+							return 0;
+						}
+
+						DELETEP(pIEG);					
+
+						errorCode = pView->cmdInsertGraphic(pFG, szFileName);
+						if (errorCode != UT_OK)
+						{
+							s_CouldNotLoadFileMessage(f, szFileName, errorCode);							
+							DELETEP(pFG);
+							return 0;
+						}
+						
+						DELETEP(pFG);
+					  }
+					else
+					{	
+						// Check if the current document is empty.
+						if (f->isDirty() || f->getFilename() ||
+							(f->getViewNumber() > 0))
+						{
+							pNewFrame = pApp->newFrame();
+							if (pNewFrame == NULL)
+							{
+								f->setStatusMessage("Could not open another window");
+								return 0;
+							}
+						}
+						else
+						{
+							pNewFrame = f;
+						}
+
+						UT_Error error = pNewFrame->loadDocument(szFileName, IEFT_Unknown);
+						if (error != UT_OK)
+						{
+							if (f != pNewFrame)
+								pNewFrame->close();
+							s_CouldNotLoadFileMessage(f, szFileName, error);
+						}
+						else
+						{
+							pNewFrame->show();
+						}
+					  }
+				}
+				else
+				{
+				}
+			}
+			DragFinish(hDrop);
+		}
 		return 0;
-	}
 
 	case WM_DESTROY:
 		return 0;
@@ -696,3 +885,22 @@ EV_Menu* XAP_Win32Frame::getMainMenu()
 {
 	return m_pWin32Menu;
 }
+
+void XAP_Win32Frame::nullUpdate() const
+{
+	MSG msg;
+	for( int i = 0 ; i < 10 ; i++ )
+	{
+		if( PeekMessage( &msg, (HWND) NULL, 0, 0, PM_REMOVE) )
+		{
+	        DispatchMessage(&msg); 
+    	} 
+	}
+}
+
+void XAP_Win32Frame::setCursor(GR_Graphics::Cursor c)
+{
+	// TODO - currently does nothing
+}
+
+
