@@ -1,5 +1,6 @@
 /* AbiWord
- * Copyright (C) 1998,1999 AbiSource, Inc.
+ * Copyright (C) Luke Jordan, Martin Sevior.
+ * BIDI Copyright (c) 2001,2002 Tomas Frydrych, Yaacov Akiba Slama
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -62,10 +63,10 @@ fl_AutoNum::fl_AutoNum(	UT_uint32 id,
 		m_bWordMultiStyle(true),
 		m_pParentItem(0)
 {
+	_setParent(pParent);
 	UT_uint32 i;
 	i =  UT_XML_strncpy( m_pszDelim, 80, lDelim);
 	i =  UT_XML_strncpy( m_pszDecimal, 80, lDecimal);
-
  	addItem(pFirst);	
 
 	m_pDoc->addList(this);
@@ -99,7 +100,7 @@ fl_AutoNum::fl_AutoNum(	UT_uint32 id,
 	UT_XML_strncpy( m_pszDecimal, 80, lDecimal);
 	if(m_iParentID != 0)
 	{
-		m_pParent = m_pDoc->getListByID(parent_id);
+		_setParent(m_pDoc->getListByID(parent_id));
 		if(m_pParent != NULL)
 			m_iLevel = m_pParent->getLevel() + 1;
 	}
@@ -112,33 +113,65 @@ void fl_AutoNum::addItem(PL_StruxDocHandle pItem)
     if(i < 0 )
     {
 		m_pItems.addItem(const_cast<void *>(pItem));
+		fixListOrder();
     }
     m_bDirty = true;
 }
 
-void fl_AutoNum::fixHierarchy(PD_Document * pDoc)
+/*!
+ * Check that this list is not a member of another list.
+ */
+void fl_AutoNum::fixHierarchy(void)
 {
 	fl_AutoNum * pParent;
-	
+	const char * pszParentID =NULL;
+#if 1
+	UT_uint32 docParentID = 0;
+	if(m_pItems.getItemCount() >0)
+	{
+		PL_StruxDocHandle sdh = (PL_StruxDocHandle) m_pItems.getNthItem(0);
+		bool bFound = m_pDoc->getAttributeFromSDH(sdh,PT_PARENTID_ATTRIBUTE_NAME,&pszParentID);
+		if(bFound)
+		{
+			docParentID= atoi(pszParentID);
+		} 
+	}
+	if((m_iID != 0) && (docParentID != 0) &&  (docParentID != m_iParentID) && (docParentID != m_iID))
+	{
+		pParent = m_pDoc->getListByID(docParentID);
+		UT_ASSERT(this != pParent);
+		if(pParent != NULL)
+		{
+			m_iParentID = docParentID;
+			m_bDirty = true;
+		}
+	}
+#endif
 	if (m_iParentID != 0)
-		pParent = pDoc->getListByID(m_iParentID);
+	{
+		pParent = m_pDoc->getListByID(m_iParentID);
+	}
 	// TODO Add error checking?
 	else
 		pParent = NULL;
 
-	m_pParent = pParent;
-
+	if(pParent != m_pParent)
+	{
+		_setParent(pParent);
+	}
+	UT_uint32 oldlevel = m_iLevel;
 	if (m_pParent)
 		m_iLevel = m_pParent->getLevel() + 1;
 	else
 		m_iLevel = 1;
-	m_bDirty = true;
+	if(oldlevel != m_iLevel)
+	{
+		m_bDirty = true;
+	}
 }
 
 fl_AutoNum::~fl_AutoNum()
 {
-	if (m_pParent && m_pParent->isEmpty())
-		DELETEP(m_pParent);
 }
 
 static PD_Document * pCurDoc;
@@ -150,10 +183,10 @@ static int compareListItems(const void* p1, const void* p2)
 	// 
 	//	pf_Frag * pf1 = (pf_Frag *) p1;
 	//	PD_Document * pDoc = pf1->getPieceTable()->getDocument();
-	PL_StruxDocHandle sdh1 = (PL_StruxDocHandle) p1;
-	PL_StruxDocHandle sdh2 = (PL_StruxDocHandle) p2;
-	PT_DocPosition pos1 = pCurDoc->getStruxPosition(sdh1);
-	PT_DocPosition pos2 = pCurDoc->getStruxPosition(sdh2);
+	PL_StruxDocHandle * sdh1 = (PL_StruxDocHandle *) p1;
+	PL_StruxDocHandle * sdh2 = (PL_StruxDocHandle *) p2;
+	PT_DocPosition pos1 = pCurDoc->getStruxPosition(*sdh1);
+	PT_DocPosition pos2 = pCurDoc->getStruxPosition(*sdh2);
 	if(pos1 < pos2)
 	{
 		return -1;
@@ -180,16 +213,26 @@ void    fl_AutoNum::markAsDirty(void)
 void    fl_AutoNum::findAndSetParentItem(void)
 {
 	if(m_iParentID == 0)
+	{		
 		return;
+	}
 	else if( m_pParent == NULL)
 	{
-		m_pParent = m_pDoc->getListByID(m_iParentID);
+		_setParent(m_pDoc->getListByID(m_iParentID));
 	}
-	if(m_pParent == NULL)
-		return;
-	//	fixListOrder();
-	//	m_pParent->fixListOrder();
-	//	m_pParent->update(0);
+	else
+	{
+		fl_AutoNum * pParent = m_pDoc->getListByID(m_iParentID);
+		if(pParent == NULL)
+		{
+			_setParent(NULL);
+		}
+	}
+
+//	pCurDoc = m_pDoc;
+//   	fixListOrder();
+//   	m_pParent->fixListOrder();
+//   	m_pParent->update(0);
 
 	if (m_pItems.getItemCount() == 0)
 	{
@@ -199,54 +242,88 @@ void    fl_AutoNum::findAndSetParentItem(void)
 	if(pCurFirst == NULL)
 		return;
 	PT_DocPosition posCur = m_pDoc->getStruxPosition(pCurFirst);
-
+	PT_DocPosition posParent = 0;
 	UT_uint32 cnt = m_pDoc->getListsCount();
-	UT_ASSERT(cnt);
+	UT_ASSERT(cnt!=0);
 	UT_uint32 iList;
 	fl_AutoNum * pClosestAuto = NULL;
 	PT_DocPosition posClosest = 0;
 	PL_StruxDocHandle pClosestItem = NULL;
-	for(iList = 0; iList < cnt; iList++) 
+	bool bReparent = false;
+	if(m_pParent != NULL)
 	{
-		fl_AutoNum * pParent = m_pDoc->getNthList(iList);
 		UT_uint32 i=0;
-		PL_StruxDocHandle pParentItem = pParent->getNthBlock(i);
-		PT_DocPosition posParent=0;
-		if(pParentItem != NULL)
+		for(i=0; i <m_pParent->getNumLabels(); i++)
 		{
-			posParent = m_pDoc->getStruxPosition(pParentItem);
+			PL_StruxDocHandle pParentItem = m_pParent->getNthBlock(i);
+			if(pParentItem != NULL)
+			{
+				posParent = m_pDoc->getStruxPosition(pParentItem);
+				if( posParent > posClosest && posParent < posCur)
+				{
+					posClosest = posParent;
+					pClosestAuto = m_pParent;
+					pClosestItem = pParentItem;
+					bReparent = true;
+				}
+			}
 		}
-		while(pParentItem != NULL && (posParent < posCur))
+	}
+//
+// Reparent this list if the first item of the parent is after the first
+// item of this list.
+//
+	if((m_pParent == NULL) || (posClosest == 0))
+	{
+		for(iList = 0; iList < cnt; iList++) 
 		{
-			i++;
-			pParentItem = pParent->getNthBlock(i);
+			fl_AutoNum * pParent = m_pDoc->getNthList(iList);
+			UT_uint32 i=0;
+			PL_StruxDocHandle pParentItem = pParent->getNthBlock(i);
+			PT_DocPosition posParent=0;
 			if(pParentItem != NULL)
 			{
 				posParent = m_pDoc->getStruxPosition(pParentItem);
 			}
-		}
-		if( i > 0)
-		{
-			i--;
-			pParentItem = pParent->getNthBlock(i);;
-			posParent = m_pDoc->getStruxPosition(pParentItem);
-			if( posParent > posClosest)
+			while(pParentItem != NULL && (posParent < posCur))
 			{
-				posClosest = posParent;
-				pClosestAuto = pParent;
-				pClosestItem = pParentItem;
+				i++;
+				pParentItem = pParent->getNthBlock(i);
+				if(pParentItem != NULL)
+				{
+					posParent = m_pDoc->getStruxPosition(pParentItem);
+				}
+			}
+			if( i > 0)
+			{
+				i--;
+				pParentItem = pParent->getNthBlock(i);;
+				posParent = m_pDoc->getStruxPosition(pParentItem);
+				if( posParent > posClosest)
+				{
+					posClosest = posParent;
+					pClosestAuto = pParent;
+					pClosestItem = pParentItem;
+					bReparent = true;
+				}
 			}
 		}
 	}
 	if(m_pParentItem != pClosestItem)
 		m_bDirty = true;
-	m_pParentItem = pClosestItem;
 	if(m_pParent != pClosestAuto)
 		m_bDirty = true;
-	m_pParent = pClosestAuto;
+	if(bReparent)
+	{
+		m_pParentItem = pClosestItem;
+		if(m_pParent != pClosestAuto)
+		{
+			_setParent(pClosestAuto);
+			_setParentID(m_pParent->getID());
+		}
+	}
 	if(m_pParent != NULL)
 	{
-		m_iParentID = m_pParent->getID();
 		m_iLevel = m_pParent->getLevel()+ 1;
 		//
 		// TODO: change all the para attributes in the list to reflect
@@ -262,7 +339,7 @@ void    fl_AutoNum::findAndSetParentItem(void)
 }
 
 void    fl_AutoNum::_getLabelstr( UT_UCSChar labelStr[], UT_uint32 * insPoint, 
-								  UT_uint32 depth, PL_StruxDocHandle pLayout)  
+								  UT_uint32 depth, PL_StruxDocHandle pLayout) const
 {
 	// This method recursively calculates a label based on the type of label
 	// of the AutoNum Class. This is output to the label string labelStr.
@@ -298,14 +375,21 @@ void    fl_AutoNum::_getLabelstr( UT_UCSChar labelStr[], UT_uint32 * insPoint,
 
 	while (p[i] && p[i] != '%' && p[i+1] != 'L')
 	{
+		// FIXME check the bounds to not overflow leftDelim
 		leftDelim[i] = p[i];
 		i++;
 	}
+	if (p[i] == '\0') {
+		UT_DEBUGMSG (("Hub: not a delim (SHOULD NOT HAPPEN)!!!\n"));
+		*insPoint = 0;
+		return;
+	}
 	leftDelim[i] = '\0';
 	i += 2;
 	rTmp = i;
 	while (p[i] || p[i] != '\0')
 	{
+		// FIXME check the bounds to not overflow rightDelim
 		rightDelim[i - rTmp] = p[i];
 		i++;
 	}
@@ -334,10 +418,10 @@ void    fl_AutoNum::_getLabelstr( UT_UCSChar labelStr[], UT_uint32 * insPoint,
 		(*insPoint) = 0;
 		return;
 	}
-	place = place + m_iStartValue;
+	place += m_iStartValue;
  
 	//	if (depth == 0 )
-	if( m_List_Type < BULLETED_LIST)
+	if(IS_NUMBERED_LIST_TYPE(m_List_Type))
 	{
 		psz = UT_XML_strlen(leftDelim);
 		for (i = 0; i < psz; i++)
@@ -392,7 +476,19 @@ void    fl_AutoNum::_getLabelstr( UT_UCSChar labelStr[], UT_uint32 * insPoint,
 			labelStr[(*insPoint)++] =  CONV_TO_UCS p[i];
 		}
 		break;
-
+#ifdef BIDI_ENABLED
+	case ARABICNUMBERED_LIST:
+		sprintf(p,"%i",place);
+		psz = UT_XML_strlen( p);
+		for(i=0; i<psz; i++)
+		{
+			labelStr[(*insPoint)++] =  (CONV_TO_UCS p[i]) + 0x0660 - (CONV_TO_UCS '0');
+		}
+		break;
+	case HEBREW_LIST:
+		dec2hebrew(labelStr,insPoint,place);
+		break;
+#endif
 	case BULLETED_LIST:
 		labelStr[(*insPoint)++] =  CONV_TO_UCS 0xb7; // was UCS_BULLET;
 		break;
@@ -455,7 +551,7 @@ void    fl_AutoNum::_getLabelstr( UT_UCSChar labelStr[], UT_uint32 * insPoint,
 	return;
 }
 
-const UT_UCSChar * fl_AutoNum::getLabel(PL_StruxDocHandle pItem) 
+const UT_UCSChar * fl_AutoNum::getLabel(PL_StruxDocHandle pItem)  const
 {
 	static UT_UCSChar label[100];
 	UT_uint32  insPoint=0;
@@ -472,7 +568,7 @@ const UT_UCSChar * fl_AutoNum::getLabel(PL_StruxDocHandle pItem)
 	}
 }
 
-UT_uint32 fl_AutoNum::getValue(PL_StruxDocHandle pItem) 
+UT_uint32 fl_AutoNum::getValue(PL_StruxDocHandle pItem) const
 {
 	return getPositionInList(pItem,0) + m_iStartValue;
 }
@@ -483,7 +579,7 @@ void fl_AutoNum::setListType(List_Type lType)
 	m_List_Type = lType;
 }
 
-bool fl_AutoNum::isDirty(void)
+bool fl_AutoNum::isDirty() const
 {
 	return m_bDirty;
 }
@@ -495,13 +591,13 @@ void fl_AutoNum::setDelim(const XML_Char * lDelim)
 	m_bDirty = true;
 }
 
-const XML_Char * fl_AutoNum::getDelim( void)
+const XML_Char * fl_AutoNum::getDelim() const
 {
 	return m_pszDelim;
 }
 
 
-const XML_Char * fl_AutoNum::getDecimal( void)
+const XML_Char * fl_AutoNum::getDecimal() const
 {
 	return m_pszDecimal;
 }
@@ -513,7 +609,7 @@ void fl_AutoNum::setDecimal(const XML_Char * lDecimal)
 	m_bDirty = true;
 }
 
-List_Type fl_AutoNum::getType(void)
+List_Type fl_AutoNum::getType() const
 {
 	return m_List_Type;
 }
@@ -531,12 +627,12 @@ void fl_AutoNum::setAsciiOffset(UT_uint32 new_asciioffset)
 	m_bDirty = true;
 }
 
-UT_uint32 fl_AutoNum::getStartValue32(void)
+UT_uint32 fl_AutoNum::getStartValue32() const
 {
 	return m_iStartValue;
 }
 
-void fl_AutoNum::insertFirstItem(PL_StruxDocHandle pItem, PL_StruxDocHandle pLast, UT_uint32 depth)
+void fl_AutoNum::insertFirstItem(PL_StruxDocHandle pItem, PL_StruxDocHandle pLast, UT_uint32 depth, bool bDoFix)
 {
 	UT_sint32 i = -1; 
 	if(m_pItems.getItemCount() > 0)
@@ -546,8 +642,10 @@ void fl_AutoNum::insertFirstItem(PL_StruxDocHandle pItem, PL_StruxDocHandle pLas
 		m_pItems.insertItemAt((void *) pItem, 0);
 		m_bDirty = true;
 	}
-
-
+	if(bDoFix)
+	{
+		fixListOrder();
+	}
 	if (m_pParent)
 	{
 		m_pParentItem = pLast;
@@ -557,7 +655,7 @@ void fl_AutoNum::insertFirstItem(PL_StruxDocHandle pItem, PL_StruxDocHandle pLas
 		_updateItems(0,NULL);
 }
 
-PL_StruxDocHandle fl_AutoNum::getParentItem(void)
+PL_StruxDocHandle fl_AutoNum::getParentItem() const
 {
 	return m_pParentItem;
 }
@@ -569,7 +667,7 @@ void fl_AutoNum::setParentItem(PL_StruxDocHandle pItem)
 	m_bDirty = true;
 }
 
-void fl_AutoNum::insertItem(PL_StruxDocHandle pItem, PL_StruxDocHandle pPrev)
+void fl_AutoNum::insertItem(PL_StruxDocHandle pItem, PL_StruxDocHandle pPrev, bool bDoFix)
 {
 	UT_sint32 ndx,i;
 	UT_ASSERT(pItem);
@@ -579,6 +677,10 @@ void fl_AutoNum::insertItem(PL_StruxDocHandle pItem, PL_StruxDocHandle pPrev)
 	m_bDirty = true;
 	ndx = m_pItems.findItem((void *) pPrev) + 1;
 	m_pItems.insertItemAt((void *) pItem, ndx);
+	if(bDoFix)
+	{
+		fixListOrder();
+	}
 	if(m_pDoc->areListUpdatesAllowed() == false)
 		return;
 
@@ -600,7 +702,7 @@ void fl_AutoNum::insertItem(PL_StruxDocHandle pItem, PL_StruxDocHandle pPrev)
 }
 
 
-void fl_AutoNum::prependItem(PL_StruxDocHandle pItem, PL_StruxDocHandle pNext)
+void fl_AutoNum::prependItem(PL_StruxDocHandle pItem, PL_StruxDocHandle pNext, bool bDoFix)
 {
 	UT_sint32 ndx;
 	UT_sint32 i;
@@ -616,6 +718,8 @@ void fl_AutoNum::prependItem(PL_StruxDocHandle pItem, PL_StruxDocHandle pNext)
 		pPrev = m_pItems.getNthItem(ndx-1);
 	}
 	m_pItems.insertItemAt((void *) pItem, ndx);
+	if(bDoFix)
+		fixListOrder(); // safety!!
 	if(m_pDoc->areListUpdatesAllowed() == false)
 		return;
 	if(pPrev != NULL)
@@ -682,7 +786,7 @@ void fl_AutoNum::removeItem(PL_StruxDocHandle pItem)
 					UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 				}
 				pAuto->setLevel(level);
-				pAuto->setParent(getParent());
+				pAuto->_setParent(getParent());
 				pAuto->m_bDirty = true;
 				pAuto->setParentItem(getParentItem());
 			}
@@ -693,12 +797,12 @@ void fl_AutoNum::removeItem(PL_StruxDocHandle pItem)
 	_updateItems(ndx,NULL);
 }
 
-UT_uint32 fl_AutoNum::getNumLabels(void)
+UT_uint32 fl_AutoNum::getNumLabels() const
 {
 	return m_pItems.getItemCount();
 }
 
-UT_sint32 fl_AutoNum::getPositionInList(PL_StruxDocHandle pItem, UT_uint32 depth) 
+UT_sint32 fl_AutoNum::getPositionInList(PL_StruxDocHandle pItem, UT_uint32 depth) const
 {
 	UT_ASSERT(m_pItems.getItemCount() > 0);
 	
@@ -712,7 +816,7 @@ UT_sint32 fl_AutoNum::getPositionInList(PL_StruxDocHandle pItem, UT_uint32 depth
 	{
 		pTmp = (PL_StruxDocHandle) m_pItems.getNthItem(i);
 		//		bOnLevel = (depth == 0);
-		fl_AutoNum * pAuto = getAutoNumFromSdh(pItem);
+		const fl_AutoNum* pAuto = getAutoNumFromSdh(pItem);
 		bOnLevel = (bool)( pAuto == this);
 		bFirstItem = (bool)(pTmp == m_pItems.getFirstItem());
 		if (pTmp == pItem)
@@ -757,9 +861,34 @@ fl_AutoNum * fl_AutoNum::getAutoNumFromSdh(PL_StruxDocHandle sdh)
 	return pAuto;
 }
 
-	   
+const fl_AutoNum* fl_AutoNum::getAutoNumFromSdh(PL_StruxDocHandle sdh) const
+{
+	UT_sint32 i;
+	const fl_AutoNum* pAuto = 0;
 
-const bool fl_AutoNum::isItem(PL_StruxDocHandle pItem) 
+	if (m_pDoc->areListUpdatesAllowed() == false)
+	{
+		if (isItem(sdh) == false)
+			return 0;
+
+		return this;
+	}
+
+	UT_sint32 numLists = m_pDoc->getListsCount();
+	for (i = 0; i < numLists; ++i)
+	{
+		pAuto = m_pDoc->getNthList(i);
+		if (pAuto->isItem(sdh))
+			break;
+	}
+
+	if (i >= numLists)
+		return 0;
+
+	return pAuto;
+}
+
+const bool fl_AutoNum::isItem(PL_StruxDocHandle pItem) const
 {
 	if (m_pItems.findItem((void *)(pItem)) == -1)
 		return false;
@@ -840,10 +969,44 @@ fl_AutoNum * fl_AutoNum::getActiveParent(void)
 	return pAutoNum;
 }
 
-void fl_AutoNum::setParent(fl_AutoNum * pParent)
+void fl_AutoNum::_setParent(fl_AutoNum * pParent)
+{
+	if(pParent == this)
+	{
+		m_pParent = NULL;
+		m_iParentID = 0;
+		m_bDirty = true;
+		return;
+	}
+	if(pParent != m_pParent)
+	{
+		char szParent[13];
+		m_pParent = pParent;
+		if(m_pParent != NULL)
+		{
+			m_iParentID  = pParent->getID();
+		}
+		else
+		{
+			m_iParentID  = 0;
+		}
+
+		sprintf(szParent,"%d",m_iParentID);
+		m_bDirty = true;
+		UT_uint32 i = 0;
+		for(i=0; i < m_pItems.getItemCount() ; i++)
+		{
+			PL_StruxDocHandle sdh = (PL_StruxDocHandle) m_pItems.getNthItem(i);
+			m_pDoc->changeStruxForLists(sdh, (const char * )szParent);
+		}
+	}
+}
+
+
+void fl_AutoNum::_setParentID(UT_uint32 iParentID)
 {
 	m_bDirty = true;
-	m_pParent = pParent;
+	m_iParentID = iParentID;
 }
 
 void fl_AutoNum::update(UT_uint32 start)
@@ -867,6 +1030,11 @@ void fl_AutoNum::_updateItems(UT_uint32 start, PL_StruxDocHandle notMe)
 	UT_sint32 j;
 	if(m_pDoc->areListUpdatesAllowed() == true)
 	{
+		//if(start == 0)
+		//	{
+		//	pCurDoc = m_pDoc;
+		//	fixListOrder();
+		//	}
 		UT_sint32 numlists = m_pDoc->getListsCount();
 		m_bUpdatingItems = true;
 		for (UT_uint32 i = start; i < m_pItems.getItemCount(); i++)
@@ -959,7 +1127,7 @@ inline UT_uint32 fl_AutoNum::_getLevelValue(fl_AutoNum * pAutoNum)
 	return pAutoNum->getValue(pBlock);
 }
 
-char *  fl_AutoNum::dec2roman(UT_sint32 value, bool lower) 
+char *  fl_AutoNum::dec2roman(UT_sint32 value, bool lower)
 {
 	UT_String roman;
 
@@ -1047,7 +1215,7 @@ char *  fl_AutoNum::dec2roman(UT_sint32 value, bool lower)
 	return rmn;
 }
 
-char * fl_AutoNum::dec2ascii(UT_sint32 value, UT_uint32 offset) 
+char * fl_AutoNum::dec2ascii(UT_sint32 value, UT_uint32 offset)
 {
 	char ascii[30];
 	UT_uint32 ndx, count, i;
@@ -1065,19 +1233,75 @@ char * fl_AutoNum::dec2ascii(UT_sint32 value, UT_uint32 offset)
 	
 	return UT_strdup(ascii);
 }
+#ifdef BIDI_ENABLED
+void fl_AutoNum::dec2hebrew(UT_UCSChar labelStr[], UT_uint32 * insPoint, UT_sint32 value)
+{
+	UT_UCSChar gHebrewDigit[22] = 
+	{
+		//   1       2       3       4       5       6       7       8       9
+		0x05D0, 0x05D1, 0x05D2, 0x05D3, 0x05D4, 0x05D5, 0x05D6, 0x05D7, 0x05D8,
+		//  10      20      30      40      50      60      70      80      90
+		0x05D9, 0x05DB, 0x05DC, 0x05DE, 0x05E0, 0x05E1, 0x05E2, 0x05E4, 0x05E6,
+		// 100     200     300     400
+		0x05E7, 0x05E8, 0x05E9, 0x05EA
+	};
 
+	bool outputSep = false;
+	UT_UCSChar digit;
+	do
+ 	{
+		UT_sint32 n3 = value % 1000;
+
+		if(outputSep)
+			labelStr[(*insPoint)++] = 0x0020; // output thousand seperator
+		outputSep = ( n3 > 0); // request to output thousand seperator next time.
+
+		// Process digit for 100 - 900
+		for(UT_sint32 n1 = 400; n1 > 0; )
+		{
+			if( n3 >= n1)
+			{
+				n3 -= n1;
+				labelStr[(*insPoint)++] = gHebrewDigit[(n1/100)-1+18];
+			} else {
+				n1 -= 100;
+			} // if
+		} // for
+
+		// Process digit for 10 - 90
+		UT_sint32 n2;
+		if( n3 >= 10 )
+		{
+			// Special process for 15 and 16
+			if(( 15 == n3 ) || (16 == n3)) {
+				// Special rule for religious reason...
+				// 15 is represented by 9 and 6, not 10 and 5
+				// 16 is represented by 9 and 7, not 10 and 6
+				n2 = 9;
+				digit = gHebrewDigit[ n2 - 1];    
+			} else {
+				n2 = n3 - (n3 % 10);
+				digit = gHebrewDigit[(n2/10)-1+9];
+			} // if
+
+			n3 -= n2;
+			labelStr[(*insPoint)++] = digit;
+		} // if
+  
+		// Process digit for 1 - 9 
+		if ( n3 > 0)
+		{
+			labelStr[(*insPoint)++] = gHebrewDigit[n3-1];
+		} // if
+		value /= 1000;
+	} while (value >= 1);
+}
+#endif
 const char ** fl_AutoNum::getAttributes(void) 
 {
 	static char  szID[15], szPid[15], szType[5], szStart[5];
 	UT_Vector va;
  
-	/*       
-			 szID = (char *)malloc(10);
-			 szPid = (char *)malloc(10);
-			 szType = (char *)malloc(5);
-			 szStart = (char *)malloc(5);
-	*/
-
 	sprintf(szID, "%i", m_iID);
 	va.addItem( (void *) "id"); va.addItem( (void *) szID);
         
@@ -1099,7 +1323,7 @@ const char ** fl_AutoNum::getAttributes(void)
 	
 	UT_uint32 counta = va.getItemCount() + 1;
 	UT_uint32 i;
-	const char ** attribs = (const char **) calloc(counta, sizeof(char *));
+	const char ** attribs = (const char **) UT_calloc(counta, sizeof(char *));
 	for (i = 0; i < va.getItemCount(); i++)
 	{
 		attribs[i] = (const char *) va[i];
