@@ -1,5 +1,6 @@
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
+ * BIDI Copyright (c) 2001,2002 Tomas Frydrych
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,11 +22,13 @@
 #include <stdlib.h>
 
 #include "ut_types.h"
+#include "ut_sleep.h"
 #include "fl_DocListener.h"
 #include "fl_Layout.h"
 #include "fl_DocLayout.h"
 #include "fl_SectionLayout.h"
 #include "fl_BlockLayout.h"
+#include "fl_Squiggles.h"
 #include "fl_AutoNum.h"
 #include "fp_Page.h"
 #include "fp_Run.h"
@@ -66,6 +69,9 @@ FL_DocLayout::FL_DocLayout(PD_Document* doc, GR_Graphics* pG)
 	m_bImSpellCheckingNow = false;
 	m_uDocBackgroundCheckReasons = 0;
 	m_iSkipUpdates = 0;
+	m_bDeletingLayout = false;
+	setLayoutIsFilling(false);
+	m_lid = 123;
 	m_pRedrawUpdateTimer = UT_Timer::static_constructor(_redrawUpdate, this, m_pG);
 	if (m_pRedrawUpdateTimer)
 	{
@@ -80,21 +86,18 @@ FL_DocLayout::FL_DocLayout(PD_Document* doc, GR_Graphics* pG)
 	// Turn off list updating until document is formatted
 
 	m_pDoc->disableListUpdates();
-	m_pDocListener = new fl_DocListener(doc, this);
-	doc->addListener(static_cast<PL_Listener *>(m_pDocListener),&m_lid);
-//
-// Put the default View color to white
-//
+
 	strncpy(m_szCurrentTransparentColor,(const char *) XAP_PREF_DEFAULT_ColorForTransparent,9);
 
 #ifdef FMT_TEST
 	m_pDocLayout = this;
 #endif
-
+	m_iRedrawCount = 0;
 }
 
 FL_DocLayout::~FL_DocLayout()
 {
+	m_bDeletingLayout = true;
 	if (m_pPrefs)
 	{
 		m_pPrefs->removeListener ( _prefsListener, this ); 
@@ -111,8 +114,17 @@ FL_DocLayout::~FL_DocLayout()
 	{
 		m_bStopSpellChecking = true;
 		m_pBackgroundCheckTimer->stop();
+#ifdef __BEOS__
+		m_pBackgroundCheckTimer->start();	// is already stopped
+											// so we don't go out of this while
+#endif
 		while(m_bImSpellCheckingNow == true)
 		{
+#ifdef __BEOS__
+			/* On BeOS, we must release the cpu as timers are
+			   run on separate threads */
+			UT_usleep(10000);
+#endif
 		}
 	}
 	
@@ -126,7 +138,18 @@ FL_DocLayout::~FL_DocLayout()
 	
 	DELETEP(m_pRedrawUpdateTimer);
 
-	UT_VECTOR_PURGEALL(fp_Page *, m_vecPages);
+	UT_sint32 count = (UT_sint32) m_vecPages.getItemCount() -1;
+	while(count >= 0)
+	{
+		fp_Page * pPage = (fp_Page *) m_vecPages.getNthItem(count);
+		if(pPage->getPrev())
+		{
+			pPage->getPrev()->setNext(NULL);
+		}
+		m_vecPages.deleteNthItem(count);
+		delete pPage;
+		count--;
+	}
 	
 	while (m_pFirstSection)
 	{
@@ -137,6 +160,55 @@ FL_DocLayout::~FL_DocLayout()
 
 	UT_HASH_PURGEDATA(GR_Font *, &m_hashFontCache, delete);
 }
+/*!
+ * This Method fills the layout structures from the PieceTable.
+ */
+void FL_DocLayout::fillLayouts(void)
+{
+	setLayoutIsFilling(true);
+	if(m_pView)
+	{
+		m_pView->setPoint(0);
+		m_pView->setLayoutIsFilling(true);
+	}
+//
+// Make a document listner to get info pumped into the layouts.
+//
+	m_pDocListener = new fl_DocListener(m_pDoc, this);
+	UT_ASSERT(m_pDocListener);
+//
+// The act of adding the listner to the document also causes the
+// the document to pump it's content into the layout classes.
+//
+	m_pDoc->addListener(static_cast<PL_Listener *>(m_pDocListener),&m_lid);
+	UT_ASSERT(m_lid != 123);
+	formatAll();
+	if(m_pView)
+	{
+		m_pView->moveInsPtTo(FV_DOCPOS_BOD);
+		m_pView->clearCursorWait();
+		m_pView->setLayoutIsFilling(false);
+	}
+
+	setLayoutIsFilling(false);
+
+	// what we want to do here is to set the default language
+	// that we're editing in
+
+	const XML_Char * doc_locale = NULL;
+	if (m_pView && XAP_App::getApp()->getPrefs()->getPrefsValue(XAP_PREF_KEY_DocumentLocale,&doc_locale))
+	{
+		if (doc_locale)
+		{
+			const XML_Char * props[3];
+			props[0] = "lang";
+			props[1] = doc_locale;
+			props[2] = 0;
+			m_pView->setCharFormat(props);
+		}
+		m_pView->notifyListeners(AV_CHG_ALL);
+	}
+}
 
 void FL_DocLayout::setView(FV_View* pView)
 {
@@ -187,6 +259,10 @@ UT_sint32 FL_DocLayout::getHeight()
 		fp_Page* p = (fp_Page*) m_vecPages.getNthItem(i);
 
 		iHeight += p->getHeight();
+		if(getView() && (getView()->getViewMode() != VIEW_PRINT))
+		{
+			iHeight = iHeight - p->getOwningSection()->getTopMargin() - p->getOwningSection()->getBottomMargin();
+		}
 	}
 
 	if (m_pG->queryProperties(GR_Graphics::DGP_SCREEN))
@@ -204,7 +280,10 @@ UT_sint32 FL_DocLayout::getHeight()
 			iHeight += fl_PAGEVIEW_MARGIN_Y * 2;
 		}
 	}
-
+	if(iHeight < 0)
+	{
+		iHeight = 0;
+	}
 	return iHeight;
 }
 
@@ -257,16 +336,16 @@ GR_Font* FL_DocLayout::findFont(const PP_AttrProp * pSpanAP,
 	}
 	// NOTE: we currently favor a readable hash key to make debugging easier
 	// TODO: speed things up with a smaller key (the three AP pointers?) 
-	char key[500];
-	sprintf(key,"%s;%s;%s;%s;%s;%s,%i",pszFamily, pszStyle, pszVariant, pszWeight, pszStretch, pszSize, iUseLayoutResolution);
-	
-	const void * pEntry = m_hashFontCache.pick(key);
+	UT_String key;
+	UT_String_sprintf(key,"%s;%s;%s;%s;%s;%s,%i",pszFamily, pszStyle, pszVariant, pszWeight, pszStretch, pszSize, iUseLayoutResolution);
+	xxx_UT_DEBUGMSG(("pszSize = %s \n",pszSize));
+	const void * pEntry = m_hashFontCache.pick(key.c_str());
 	if (!pEntry)
 	{
 		// TODO -- note that we currently assume font-family to be a single name,
 		// TODO -- not a list.  This is broken.
 
-		if(iUseLayoutResolution)
+		if(iUseLayoutResolution == FIND_FONT_AT_LAYOUT_RESOLUTION)
 		{
 			m_pG->setLayoutResolutionMode(true);
 		}
@@ -275,14 +354,13 @@ GR_Font* FL_DocLayout::findFont(const PP_AttrProp * pSpanAP,
 		UT_ASSERT(pFont);
 
 		// add it to the cache
-		m_hashFontCache.insert(key, 
+		m_hashFontCache.insert(key.c_str(), 
 				       (void *)pFont);
 	}
 	else
 	{
 		pFont = (GR_Font*) pEntry;
 	}
-
 	return pFont;
 }
 
@@ -312,13 +390,13 @@ GR_Font* FL_DocLayout::findFont(const PP_AttrProp * pSpanAP,
 	}
 	// NOTE: we currently favor a readable hash key to make debugging easier
 	// TODO: speed things up with a smaller key (the three AP pointers?) 
-	char key[500];
+	UT_String key;
 	if(pszField!="NULL" && pszField != NULL && isField==true)
 	{
 		pszFamily = pszField;
 	}
-	sprintf(key,"%s;%s;%s;%s;%s;%s,%i",pszFamily, pszStyle, pszVariant, pszWeight, pszStretch, pszSize, iUseLayoutResolution);
-	const void *pEntry = m_hashFontCache.pick(key);
+	UT_String_sprintf(key,"%s;%s;%s;%s;%s;%s,%i",pszFamily, pszStyle, pszVariant, pszWeight, pszStretch, pszSize, iUseLayoutResolution);
+	const void *pEntry = m_hashFontCache.pick(key.c_str());
 	if (!pEntry)
 	{
 		// TODO -- note that we currently assume font-family to be a single name,
@@ -333,7 +411,7 @@ GR_Font* FL_DocLayout::findFont(const PP_AttrProp * pSpanAP,
 		UT_ASSERT(pFont);
 
 		// add it to the cache
-		m_hashFontCache.insert(key, 
+		m_hashFontCache.insert(key.c_str(), 
 				       (void *)pFont);
 	}
 	else
@@ -366,6 +444,16 @@ UT_uint32 FL_DocLayout::countPages()
 	return m_vecPages.getItemCount();
 }
 
+UT_sint32 FL_DocLayout::findPage(fp_Page * pPage)
+{
+	UT_sint32 count = m_vecPages.getItemCount();
+	if(count < 1)
+	{
+		return -1;
+	}
+	return m_vecPages.findItem((void *) pPage);
+}
+
 fp_Page* FL_DocLayout::getNthPage(int n)
 {
 	UT_ASSERT(m_vecPages.getItemCount() > 0);
@@ -414,8 +502,10 @@ void FL_DocLayout::deletePage(fp_Page* pPage, bool bDontNotify /* default false
 	// let the view know that we deleted a page,
 	// so that it can update the scroll bar ranges
 	// and whatever else it needs to do.
-
-	if (m_pView && !bDontNotify)
+    //
+    // Check for point > 0 to allow multi-threaded loads
+    //
+	if (m_pView && !bDontNotify && m_pView->getPoint() > 0)
 	{
 		m_pView->notifyListeners(AV_CHG_PAGECOUNT);
 	}
@@ -433,6 +523,7 @@ fp_Page* FL_DocLayout::addNewPage(fl_DocSectionLayout* pOwner)
 	{
 		pLastPage = NULL;
 	}
+
 	fp_Page* pPage = new fp_Page(	this,
 									m_pView,
 									m_pDoc->m_docPageSize,
@@ -445,12 +536,13 @@ fp_Page* FL_DocLayout::addNewPage(fl_DocSectionLayout* pOwner)
 	}
 	pPage->setPrev(pLastPage);
 	m_vecPages.addItem(pPage);
+	pOwner->addOwnedPage(pPage);
 
 	// let the view know that we created a new page,
 	// so that it can update the scroll bar ranges
 	// and whatever else it needs to do.
 
-	if (m_pView && m_pView->shouldScreenUpdateOnGeneralUpdate()) // skip this if rebuilding
+	if (m_pView && m_pView->shouldScreenUpdateOnGeneralUpdate() && m_pView->getPoint() > 0) // skip this if rebuilding or if we're loading a document
 	{
 		m_pView->notifyListeners(AV_CHG_PAGECOUNT);
 	}
@@ -468,7 +560,7 @@ fp_Page* FL_DocLayout::addNewPage(fl_DocSectionLayout* pOwner)
 fl_BlockLayout* FL_DocLayout::findBlockAtPosition(PT_DocPosition pos)
 {
 	fl_BlockLayout* pBL = NULL;
-	PL_StruxFmtHandle sfh;
+	PL_StruxFmtHandle sfh = 0;
 
 	PT_DocPosition posEOD;
 	bool bRes;
@@ -488,6 +580,9 @@ fl_BlockLayout* FL_DocLayout::findBlockAtPosition(PT_DocPosition pos)
 	if (bRes)
 	{
 		fl_Layout * pL = (fl_Layout *)sfh;
+		if(!pL)
+			return NULL;
+		
 		switch (pL->getType())
 		{
 		case PTX_Block:
@@ -498,6 +593,7 @@ fl_BlockLayout* FL_DocLayout::findBlockAtPosition(PT_DocPosition pos)
 		default:
 			UT_ASSERT(UT_SHOULD_NOT_HAPPEN); 
 			// We asked for a block, and we got a section.  Bad
+			return NULL;
 		}
 	}
 	else
@@ -513,6 +609,30 @@ fl_BlockLayout* FL_DocLayout::findBlockAtPosition(PT_DocPosition pos)
 		if(pView && pView->isHdrFtrEdit())
 		{
 			pShadow = pView->getEditShadow();
+//
+// We might actually be in the other HdrFtr is the point got here from an undo!
+// Check for this.
+//
+			if(!pShadow->getHdrFtrSectionLayout()->isPointInHere(pos))
+			{
+				fl_HdrFtrSectionLayout * pHF = (fl_HdrFtrSectionLayout *) pBL->getSectionLayout();
+				if(pHF->isPointInHere(pos))
+				{
+					pShadow = pHF->getFirstShadow();
+					pView->clearHdrFtrEdit();
+					pView->setHdrFtrEdit(pShadow);
+					pBL = pShadow->findBlockAtPosition(pos);
+					return pBL;
+				}
+				// Ok, we're really confused now, point is nowhere to be found.
+				// It might be OK if pos-1 is in here, though...
+				if (!pShadow->getHdrFtrSectionLayout()->isPointInHere(pos-1))
+				{
+					UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+				}
+				pBL = NULL;
+			}   
+					
 		}
 		else
 		{
@@ -587,7 +707,10 @@ void FL_DocLayout::formatAll()
 	while (pSL)
 	{
 		pSL->format();
-		
+		if(pSL->getType() == FL_SECTION_DOC)
+		{
+			static_cast<fl_DocSectionLayout *>(pSL)->checkAndRemovePages();
+		}
 		pSL = pSL->getNext();
 	}
 }
@@ -739,9 +862,18 @@ void FL_DocLayout::updateColor()
 
 #define BACKGROUND_CHECK_MSECS 100
 
-void FL_DocLayout::_toggleAutoSpell(bool bSpell)
+/*!
+ Toggle auto spell-checking state
+ \param bSpell True if spell-checking should be enabled, false otherwise
+ When disabling spelling, all squiggles are deleted.
+ When enabling spelling, force a full check of the document.
+*/
+void
+FL_DocLayout::_toggleAutoSpell(bool bSpell)
 {
 	bool bOldAutoSpell = getAutoSpellCheck();
+
+	// Add reason to background checker
 	if (bSpell)
 	{
 		addBackgroundCheckReason(bgcrSpelling);
@@ -751,11 +883,12 @@ void FL_DocLayout::_toggleAutoSpell(bool bSpell)
 		removeBackgroundCheckReason(bgcrSpelling);
 	}
 
-	UT_DEBUGMSG(("FL_DocLayout::_toggleAutoSpell (%s)\n", bSpell ? "true" : "false" ));
+	xxx_UT_DEBUGMSG(("FL_DocLayout::_toggleAutoSpell (%s)\n", 
+					 bSpell ? "true" : "false" ));
 
 	if (bSpell)
 	{
-		// recheck the whole doc
+		// When enabling, recheck the whole document
 		fl_DocSectionLayout * pSL = getFirstSection();
 		while (pSL)
 		{
@@ -772,7 +905,7 @@ void FL_DocLayout::_toggleAutoSpell(bool bSpell)
 	}
 	else
 	{
-		// remove the squiggles, too
+		// Disabling, so remove the squiggles too
 		fl_DocSectionLayout * pSL = getFirstSection();
 		while (pSL)
 		{
@@ -780,17 +913,20 @@ void FL_DocLayout::_toggleAutoSpell(bool bSpell)
 			while (b)
 			{
 				b->removeBackgroundCheckReason(bgcrSpelling);
-				b->_purgeSquiggles();
+				b->getSquiggles()->deleteAll();
 				b = b->getNext();
 			}
 			pSL = (fl_DocSectionLayout *) pSL->getNext();
 		}
 		if (bOldAutoSpell)
 		{
-			// If we're here, it was set to TRUE before but now it is being set
-			// to FALSE. This means that it is the user setting it. That's good.
+			// If we're here, it was set to TRUE before but now it is
+			// being set to FALSE. This means that it is the user
+			// setting it. That's good.
 			m_pView->draw(NULL);
-			// A pending word would be bad. Not sure why it's not ignored once autospell is off, but for now it should definattely be annulled.
+			// A pending word would be bad. Not sure why it's not
+			// ignored once autospell is off, but for now it should
+			// definitely be annulled.
 			setPendingWordForSpell(NULL, NULL);
 		}
 	}
@@ -811,67 +947,82 @@ void FL_DocLayout::_toggleAutoSmartQuotes(bool bSQ)
 	UT_DEBUGMSG(("FL_DocLayout::_toggleAutoSmartQuotes(%s)\n", bSQ ? "true" : "false" ));
 }
 
-void FL_DocLayout::_backgroundCheck(UT_Worker * pWorker)
+/*!
+ Do background spell-check
+ \param pWorker Worker object
+ \note This is a static callback method and does not have a 'this' pointer.
+*/
+void
+FL_DocLayout::_backgroundCheck(UT_Worker * pWorker)
 {
 	UT_ASSERT(pWorker);
 
-	// this is a static callback method and does not have a 'this' pointer.
-
+	// Get the doclayout
 	FL_DocLayout * pDocLayout = (FL_DocLayout *) pWorker->getInstanceData();
 	UT_ASSERT(pDocLayout);
 
+	// Win32 timers can fire prematurely on asserts (the dialog's
+	// message pump releases the timers)
 	if (!pDocLayout->m_pView)
 	{
-		// Win32 timers can fire prematurely on asserts
-		// (the dialog's message pump releases the timers)
 		return;
 	}
-//
-// Don't spell check while printing!
-//
+
+	// Don't spell check while printing!
 	if(pDocLayout->m_pG->queryProperties(GR_Graphics::DGP_PAPER))
 	{
 		return;
 	}
-	if(pDocLayout->m_bStopSpellChecking == true || pDocLayout->m_bImSpellCheckingNow == true)
-	{
-		return;
-	}
-	// Code added to hold spell checks during block insertions
 
-	if(pDocLayout->m_pDoc->isPieceTableChanging() == true)
+	// Don't spell check if disabled, or already happening
+	if(pDocLayout->m_bStopSpellChecking || pDocLayout->m_bImSpellCheckingNow || pDocLayout->isLayoutFilling())
 	{
 		return;
 	}
+
+	// Code added to hold spell checks during block insertions
+	if(pDocLayout->m_pDoc->isPieceTableChanging())
+	{
+		return;
+	}
+
+	// Don't spell check while a redrawupdate is happening either...
+	PD_Document * pDoc = pDocLayout->getDocument();
+	if(pDoc->isRedrawHappenning())
+	{
+		return;
+	}
+
+	// Flag that spell checking is in action.
+	// Note: this is not a good way to do mutual exclusion!
 	pDocLayout->m_bImSpellCheckingNow = true;
 
-	// prevent getting a new timer hit before we've finished this one by
-	// temporarily disabling the timer
-	//pDocLayout->m_pBackgroundCheckTimer->stop();
-
+	// Find vector of blocks to check.
 	UT_Vector* vecToCheck = &pDocLayout->m_vecUncheckedBlocks;
 	UT_ASSERT(vecToCheck);
 
 	UT_uint32 i = vecToCheck->getItemCount();
-
 	if (i > 0)
 	{
+		// Check each block in the queue
 		fl_BlockLayout *pB = (fl_BlockLayout *) vecToCheck->getFirstItem();
-
 		if (pB != NULL)
 		{
-			for (unsigned int bitdex=0; bitdex<8*sizeof(pB->m_uBackgroundCheckReasons); ++bitdex)
+			// This looping seems like a lot of wasted effort when we
+			// don't define meaning for most of the bits, but it's
+			// small effort compared to all that squiggle stuff that
+			// goes on for the spelling stuff.
+			for (UT_uint32 bitdex = 0; 
+				 bitdex < 8*sizeof(pB->m_uBackgroundCheckReasons);
+				 bitdex++)
 			{
-				// This looping seems like a lot of wasted effort when we
-				// don't define meaning for most of the bits, but it's small
-				// effort compared to all that squiggle stuff that goes on
-				// for the spelling stuff.
 				UT_uint32 mask;
 				mask = (1 << bitdex);
 				if (pB->hasBackgroundCheckReason(mask))
 				{
-					//	note that we remove this reason from queue before checking it
-					//	(otherwise asserts could trigger redundant recursive calls)
+					// Note that we remove this reason from queue
+					// before checking it (otherwise asserts could
+					// trigger redundant recursive calls)
 					pB->removeBackgroundCheckReason(mask);
 					switch (mask)
 					{
@@ -889,6 +1040,8 @@ void FL_DocLayout::_backgroundCheck(UT_Worker * pWorker)
 					}
 				}
 			}
+			// Delete block from queue if there are no more reasons
+			// for checking it.
 			if (!pB->m_uBackgroundCheckReasons)
 			{
 				vecToCheck->deleteNthItem(0);
@@ -898,27 +1051,30 @@ void FL_DocLayout::_backgroundCheck(UT_Worker * pWorker)
 	}
 	else
 	{ 
-//
-// No blocks to spellcheck so stop the idle/timer. Otherwise we consume 100% CPU.
-//
+		// No blocks to spellcheck so stop the idle/timer. Otherwise
+		// we consume 100% CPU.
 		pDocLayout->m_pBackgroundCheckTimer->stop();
 	}
-		
-	if (i != 0 && pDocLayout->m_bStopSpellChecking == false)
-	{
-		// restart timer unless it's not needed any more
-		//pDocLayout->m_pBackgroundCheckTimer->start();
-	}
+
 	pDocLayout->m_bImSpellCheckingNow = false;
 }
 
-void FL_DocLayout::queueBlockForBackgroundCheck(UT_uint32 reason, fl_BlockLayout *pBlock, bool bHead)
+/*!
+ Enqueue block for background spell-checking
+ \param iReason Reason for checking the block FIXME - enum?
+ \param pBlock Block to enqueue
+ \param bHead When true, insert block at head of queue
+
+ This routine queues up blocks for timer-driven spell checking, etc.  
+ By default, this is a FIFO queue, but it can be explicitly 
+ reprioritized by setting bHead to true.
+*/
+void
+FL_DocLayout::queueBlockForBackgroundCheck(UT_uint32 iReason,
+										   fl_BlockLayout *pBlock,
+										   bool bHead)
 {
-	/*
-	  This routine queues up blocks for timer-driven spell checking, etc.  
-	  By default, this is a FIFO queue, but it can be explicitly 
-	  reprioritized by setting bHead == true.  
-	*/
+	// If there's no timer running, start one
 	if (!m_pBackgroundCheckTimer)
 	{
 	    int inMode = UT_WorkerFactory::IDLE | UT_WorkerFactory::TIMER;
@@ -929,9 +1085,11 @@ void FL_DocLayout::queueBlockForBackgroundCheck(UT_uint32 reason, fl_BlockLayout
 	    UT_ASSERT(m_pBackgroundCheckTimer);
 	    UT_ASSERT(outMode != UT_WorkerFactory::NONE);
 
+		// If the worker is working on a timer instead of in the idle
+		// time, set the frequency of the checks.
 	    if ( UT_WorkerFactory::TIMER == outMode )
 		{
-		// this is really a timer, so it's safe to static_cast it
+			// this is really a timer, so it's safe to static_cast it
 			static_cast<UT_Timer*>(m_pBackgroundCheckTimer)->set(BACKGROUND_CHECK_MSECS);
 		}
 #if 1
@@ -949,17 +1107,18 @@ void FL_DocLayout::queueBlockForBackgroundCheck(UT_uint32 reason, fl_BlockLayout
 	}
 #endif
 
+	// Set debug flash reason on block if it is set
 	if (hasBackgroundCheckReason(bgcrDebugFlash))
 	{
 		pBlock->addBackgroundCheckReason(bgcrDebugFlash);
 	}
-	pBlock->addBackgroundCheckReason(reason);
+	pBlock->addBackgroundCheckReason(iReason);
 
 	UT_sint32 i = m_vecUncheckedBlocks.findItem(pBlock);
-
 	if (i < 0)
 	{
-		// add it
+		// Add block if it's not already in the queue. Add it either
+		// at the head, or at the tail.
 		if (bHead)
 			m_vecUncheckedBlocks.insertItemAt(pBlock, 0);
 		else
@@ -967,96 +1126,152 @@ void FL_DocLayout::queueBlockForBackgroundCheck(UT_uint32 reason, fl_BlockLayout
 	}
 	else if (bHead)
 	{
-		// bubble it to the start
+		// Block is already in the queue, bubble it to the start
 		m_vecUncheckedBlocks.deleteNthItem(i);
 		m_vecUncheckedBlocks.insertItemAt(pBlock, 0);
 	}
 }
 
-void FL_DocLayout::dequeueBlockForBackgroundCheck(fl_BlockLayout *pBlock)
-{
-	UT_sint32 i = m_vecUncheckedBlocks.findItem(pBlock);
+/*!
+ Remove block from background checking queue
+ \param pBlock Block to remove
 
-	if (i>=0)
+ When the last block is removed from the queue, the background timer
+ is stopped. The function does not return before the background
+ spell-checking timer has stopped.
+*/
+bool
+FL_DocLayout::dequeueBlockForBackgroundCheck(fl_BlockLayout *pBlock)
+{
+	bool bRes = false;
+
+	// Remove block from queue if it's found there
+	UT_sint32 i = m_vecUncheckedBlocks.findItem(pBlock);
+	if (i >= 0)
 	{
 		m_vecUncheckedBlocks.deleteNthItem(i);
+		bRes = true;
 	}
 
-	// when queue is empty, kill timer
+	// When queue is empty, kill timer
 	if (m_vecUncheckedBlocks.getItemCount() == 0)
 	{
 		m_bStopSpellChecking = true;
 		if(m_pBackgroundCheckTimer)
 		{
 			m_pBackgroundCheckTimer->stop();
+			// Wait for checking to complete before returning.
 			while(m_bImSpellCheckingNow == true)
 			{
 			}
 		}
 	}
+
+	return bRes;
 }
 
-void FL_DocLayout::setPendingWordForSpell(fl_BlockLayout *pBlock, fl_PartOfBlock* pWord)
+/*!
+  Mark a region of a block to be spell checked
+  \param pBlock Block
+  \param pWord  Region
+
+  If called with NULL arguments, any prior marked region will be
+  freed. Callers must reuse pWord (by calling getPendingWordForSpell)
+  when set.  
+*/
+void 
+FL_DocLayout::setPendingWordForSpell(fl_BlockLayout *pBlock,
+									 fl_PartOfBlock* pWord)
 {
+	// Return if matching the existing marked region
 	if ((pBlock == m_pPendingBlockForSpell) && 
 		(pWord == m_pPendingWordForSpell))
 		return;
 
-	UT_ASSERT(!m_pPendingBlockForSpell || !pBlock);
+	// Assert an existing pWord allocation is reused
+	UT_ASSERT(!m_pPendingBlockForSpell || !pBlock
+			  || m_pPendingWordForSpell == pWord);
 
+	// Check for valid arguments
 	if (pBlock && m_pPendingBlockForSpell && m_pPendingWordForSpell)
 	{
 		UT_ASSERT(pWord);
 	}
 
-	// when clobbering prior POB, make sure we don't leak it
-	DELETEP(m_pPendingWordForSpell);
+	if (m_pPendingWordForSpell && (m_pPendingWordForSpell != pWord))
+	{
+		// When clobbering prior POB, make sure we don't leak it
+		DELETEP(m_pPendingWordForSpell);
+	}
 
 	m_pPendingBlockForSpell = pBlock;
 	m_pPendingWordForSpell = pWord;
 }
 
-bool FL_DocLayout::checkPendingWordForSpell(void)
+/*!
+ Spell-check pending word
+ \result True if word checked, false otherwise
+ If a word is pending, spell-check it.
+
+ \note This function used to exit if PT was changing - but that
+       prevents proper squiggle behavior during undo, so the check has
+       been removed. This means that the pending word POB must be
+       updated to reflect the PT changes before the IP is moved.
+*/
+bool
+FL_DocLayout::checkPendingWordForSpell(void)
 {
 	bool bUpdate = false;
 
+	xxx_UT_DEBUGMSG(("FL_DocLayout::checkPendingWordForSpell\n"));
+
 	if (!m_pPendingBlockForSpell)
 		return bUpdate;
 
-	if(m_pDoc->isPieceTableChanging() == true)
-	{
-		return bUpdate;
-	}
-
-	// check pending word
+	// Check pending word
 	UT_ASSERT(m_pPendingWordForSpell);
 	bUpdate = m_pPendingBlockForSpell->checkWord(m_pPendingWordForSpell);
 
 	m_pPendingWordForSpell = NULL;	// NB: already freed by checkWord
 
-	// not pending any more
+	// Not pending any more
 	setPendingWordForSpell(NULL, NULL);
 
 	return bUpdate;
 }
 
-bool FL_DocLayout::isPendingWordForSpell(void) const
+/*!
+ Is a word pending for spelling
+ \return True if a word is pending, false otherwise
+*/
+bool
+FL_DocLayout::isPendingWordForSpell(void) const
 {
 	return (m_pPendingBlockForSpell ? true : false);
 }
 
-bool	FL_DocLayout::touchesPendingWordForSpell(fl_BlockLayout *pBlock, 
-												 UT_uint32 iOffset, 
-												 UT_sint32 chg) const
+/*!
+ Determine if position touches the pending word for spelling
+ \param pBLock Block of position
+ \param iOffset Offset in block
+ \param chg  FIXME
+ \return True if position touches pending word, false otherwise
+
+ FIXME: why this function/chg? Caller uses result for what?
+*/
+bool
+FL_DocLayout::touchesPendingWordForSpell(fl_BlockLayout *pBlock, 
+										 UT_sint32 iOffset, 
+										 UT_sint32 chg) const
 {
 	UT_uint32 len = (chg < 0) ? -chg : 0;
 
+	UT_ASSERT(pBlock);
+
 	if (!m_pPendingBlockForSpell)
 		return false;
 
-	UT_ASSERT(pBlock);
-
-	// are we in the same block?
+	// Are we in the same block?
 	if (m_pPendingBlockForSpell != pBlock)
 		return false;
 
@@ -1064,6 +1279,7 @@ bool	FL_DocLayout::touchesPendingWordForSpell(fl_BlockLayout *pBlock,
 
 	return m_pPendingWordForSpell->doesTouch(iOffset, len);
 }
+
 /*!
  * This method appends a DocSectionLayout onto the linked list of SectionLayout's
  * and updates the m_pFirstSection and m_pLastSection member variables 
@@ -1231,19 +1447,46 @@ fl_DocSectionLayout* FL_DocLayout::findSectionForHdrFtr(const char* pszHdrFtrID)
 	while (pDocSL)
 	{
 		pszAtt = pDocSL->getAttribute("header");
-		if (
-			pszAtt
-			&& (0 == UT_stricmp(pszAtt, pszHdrFtrID))
-			)
+		if ( pszAtt	&& (0 == UT_stricmp(pszAtt, pszHdrFtrID)))
 		{
 			return pDocSL;
 		}
 		
 		pszAtt = pDocSL->getAttribute("footer");
-		if (
-			pszAtt
-			&& (0 == UT_stricmp(pszAtt, pszHdrFtrID))
-			)
+		if (pszAtt && (0 == UT_stricmp(pszAtt, pszHdrFtrID)))
+		{
+			return pDocSL;
+		}
+		pszAtt = pDocSL->getAttribute("header-even");
+		if ( pszAtt	&& (0 == UT_stricmp(pszAtt, pszHdrFtrID)))
+		{
+			return pDocSL;
+		}
+		
+		pszAtt = pDocSL->getAttribute("footer-even");
+		if (pszAtt && (0 == UT_stricmp(pszAtt, pszHdrFtrID)))
+		{
+			return pDocSL;
+		}
+		pszAtt = pDocSL->getAttribute("header-last");
+		if ( pszAtt	&& (0 == UT_stricmp(pszAtt, pszHdrFtrID)))
+		{
+			return pDocSL;
+		}
+		
+		pszAtt = pDocSL->getAttribute("footer-last");
+		if (pszAtt && (0 == UT_stricmp(pszAtt, pszHdrFtrID)))
+		{
+			return pDocSL;
+		}
+		pszAtt = pDocSL->getAttribute("header-first");
+		if ( pszAtt	&& (0 == UT_stricmp(pszAtt, pszHdrFtrID)))
+		{
+			return pDocSL;
+		}
+		
+		pszAtt = pDocSL->getAttribute("footer-first");
+		if (pszAtt && (0 == UT_stricmp(pszAtt, pszHdrFtrID)))
 		{
 			return pDocSL;
 		}
@@ -1376,7 +1619,12 @@ fl_DocSectionLayout* FL_DocLayout::findSectionForEndnotes(const char* pszEndnote
 	const XML_Char * pszTransparentColor = NULL;
 	pPrefs->getPrefsValue((const XML_Char *)XAP_PREF_KEY_ColorForTransparent,&pszTransparentColor);
 	if(UT_strcmp(pszTransparentColor,pDocLayout->m_szCurrentTransparentColor) != 0)
-		pDocLayout->updateColor();
+	{   
+		if(pDocLayout->getView() && (pDocLayout->getView()->getPoint() > 0))
+		{
+			pDocLayout->updateColor();
+		}
+	}
 }
 
 void FL_DocLayout::recheckIgnoredWords()
@@ -1404,12 +1652,13 @@ void FL_DocLayout::_redrawUpdate(UT_Worker * pWorker)
 	FL_DocLayout * pDocLayout = (FL_DocLayout *) pWorker->getInstanceData();
 	UT_ASSERT(pDocLayout);
 
-	if (!pDocLayout->m_pView)
+	if (!pDocLayout->m_pView || pDocLayout->isLayoutFilling())
 	{
 		// Win32 timers can fire prematurely on asserts
 		// (the dialog's message pump releases the timers)
 		return;
 	}
+	xxx_UT_DEBUGMSG(("SEVIOR: _redraw update \n"));
 //
 // Check if we're in the middle of a PieceTable change. If so don't redraw!
 //
@@ -1477,6 +1726,7 @@ void FL_DocLayout::_redrawUpdate(UT_Worker * pWorker)
 // we've finished
 //
 	pDoc->setRedrawHappenning(false);
+	pDocLayout->m_iRedrawCount++;
 }
 
 void FL_DocLayout::setPendingSmartQuote(fl_BlockLayout *bl, UT_uint32 of)
@@ -1793,6 +2043,20 @@ static struct sqTable sqTable_en[] =
 	{sqDONTCARE, 0, sqDONTCARE, UCS_UNKPUNK}  // signals end of table
 };
 
+#ifdef BIDI_ENABLED
+static void s_swapQuote(UT_UCSChar & c)
+{
+    if(c == UCS_LQUOTE)
+        c = UCS_RQUOTE;
+    else if(c == UCS_RQUOTE)
+        c = UCS_LQUOTE;
+    else if(c == UCS_LDBLQUOTE)
+        c = UCS_RDBLQUOTE;
+    else if(c == UCS_RDBLQUOTE)
+        c = UCS_LDBLQUOTE;
+}
+#endif
+
 void FL_DocLayout::considerSmartQuoteCandidateAt(fl_BlockLayout *block, UT_uint32 offset)
 {
 	if (!block) 
@@ -1904,6 +2168,16 @@ void FL_DocLayout::considerSmartQuoteCandidateAt(fl_BlockLayout *block, UT_uint3
 		UT_DEBUGMSG(("before %d, after %d, replace %x\n", before, after, replacement));
 		if (replacement != UCS_UNKPUNK)
 		{
+#ifdef BIDI_ENABLED
+            UT_sint32 s1,s2,s3,s4,s5;
+            bool b1;
+            fp_Run * pThisRun = block->findPointCoords(block->getPosition() + offset,false,s1,s2,s3,s4,s5,b1);
+            
+            xxx_UT_DEBUGMSG(("pThisRun [0x%x], vis dir. %d\n", pThisRun, pThisRun->getVisDirection()));
+            
+            if(pThisRun && pThisRun->getVisDirection() == FRIBIDI_TYPE_RTL)
+                s_swapQuote(replacement);
+#endif
 			// your basic emacs (save-excursion...)  :-)
 			PT_DocPosition saved_pos, quotable_at;
 			saved_pos = m_pView->getPoint();
@@ -1938,6 +2212,11 @@ void FL_DocLayout::notifyBlockIsBeingDeleted(fl_BlockLayout *pBlock)
 	{
 		m_pPendingBlockForSmartQuote = NULL;
 	}
+	UT_sint32 loc = m_vecUncheckedBlocks.findItem((void *) pBlock);
+	if(loc >= 0)
+	{
+		m_vecUncheckedBlocks.deleteNthItem(loc);
+	}
 }
 
 inline fl_AutoNum * FL_DocLayout::getListByID(UT_uint32 id) const
