@@ -82,7 +82,7 @@ void signalWrapper(int sig_num);
 #include <TranslationUtils.h>
 #include <DataIO.h>
 
-#define SPLASH_UP_TIME	5				// seconds
+#define SPLASH_UP_TIME	1				// seconds
 
 extern unsigned char g_pngSplash[];             // see ap_wp_Splash.cpp
 extern unsigned long g_pngSplash_sizeof;        // see ap_wp_Splash.cpp
@@ -186,9 +186,8 @@ AP_BeOSApp::~AP_BeOSApp(void)
 
 	DELETEP(m_pStringSet);
 	DELETEP(m_pClipboard);
-
-	IE_Exp::unregisterAllExporters ();
-	IE_Imp::unregisterAllImporters ();
+	
+	IE_ImpExp_UnRegisterXP ();
 }
 
 static bool s_createDirectoryIfNecessary(const char * szDir)
@@ -263,7 +262,7 @@ bool AP_BeOSApp::initialize(void)
 		getPrefsValue(AP_PREF_KEY_SpellCheckWordList,&szSpellCheckWordList);
 		UT_ASSERT((szSpellCheckWordList) && (*szSpellCheckWordList));
 		
-		char * szPathname = (char *)calloc(sizeof(char),strlen(szISpellDirectory)+strlen(szSpellCheckWordList)+2);
+		char * szPathname = (char *)UT_calloc(sizeof(char),strlen(szISpellDirectory)+strlen(szSpellCheckWordList)+2);
 		UT_ASSERT(szPathname);
 		
 		sprintf(szPathname,"%s%s%s",
@@ -303,7 +302,7 @@ bool AP_BeOSApp::initialize(void)
 			getPrefsValueDirectory(true,AP_PREF_KEY_StringSetDirectory,&szDirectory);
 			UT_ASSERT((szDirectory) && (*szDirectory));
 
-			char * szPathname = (char *)calloc(sizeof(char),strlen(szDirectory)+strlen(szStringSet)+100);
+			char * szPathname = (char *)UT_calloc(sizeof(char),strlen(szDirectory)+strlen(szStringSet)+100);
 			UT_ASSERT(szPathname);
 
 			sprintf(szPathname,"%s%s%s.strings",
