@@ -181,8 +181,10 @@ UT_Error AP_BeOSFrame::_showDocument(UT_uint32 iZoom)
                 pOldDoc = ((AP_FrameData*)m_pData)->m_pDocLayout->getDocument();
         }
 
+		delete ((AP_FrameData*)m_pData)->m_pDocLayout;
         REPLACEP(((AP_FrameData*)m_pData)->m_pG, pG);
-        REPLACEP(((AP_FrameData*)m_pData)->m_pDocLayout, pDocLayout);
+        ((AP_FrameData*)m_pData)->m_pDocLayout = pDocLayout;
+
         if (pOldDoc != m_pDoc) {
                 UNREFP(pOldDoc);
         }       
@@ -202,9 +204,16 @@ UT_Error AP_BeOSFrame::_showDocument(UT_uint32 iZoom)
 	// views, like we do for all the other objects.  We also do not
 	// allocate the TopRuler, LeftRuler  here; that is done as the
 	// frame is created.
-	((AP_FrameData*)m_pData)->m_pTopRuler->setView(pView);
-	((AP_FrameData*)m_pData)->m_pLeftRuler->setView(pView);
-	((AP_FrameData*)m_pData)->m_pStatusBar->setView(pView);
+	if ( ((AP_FrameData*)m_pData)->m_bShowRuler )
+	{
+		if ( ((AP_FrameData*)m_pData)->m_pTopRuler )
+			((AP_FrameData*)m_pData)->m_pTopRuler->setView(pView, iZoom);
+		if ( ((AP_FrameData*)m_pData)->m_pLeftRuler )
+			((AP_FrameData*)m_pData)->m_pLeftRuler->setView(pView, iZoom);
+	}                      
+
+        if ( ((AP_FrameData*)m_pData)->m_pStatusBar )
+		((AP_FrameData*)m_pData)->m_pStatusBar->setView(pView);  
 
 	pView->setInsertMode(((AP_FrameData*)m_pData)->m_bInsertMode);
     ((FV_View *) m_pView)->setShowPara(((AP_FrameData*)m_pData)->m_bShowPara);
@@ -220,6 +229,7 @@ UT_Error AP_BeOSFrame::_showDocument(UT_uint32 iZoom)
 	setYScrollRange();
 	updateTitle();
 
+	pDocLayout->fillLayouts();
 	if (point != 0)
 		((FV_View *) m_pView)->moveInsPtTo(point);
 
@@ -423,7 +433,9 @@ void AP_BeOSFrame::killFrameData(void)
 	m_pData = NULL;
 }
 
-UT_Error AP_BeOSFrame::_loadDocument(const char * szFilename, IEFileType ieft)
+UT_Error AP_BeOSFrame::_loadDocument(const char * szFilename, IEFileType ieft,
+				     bool createNew)
+	
 {
 	// are we replacing another document?
 	if (m_pDoc)
@@ -445,14 +457,32 @@ UT_Error AP_BeOSFrame::_loadDocument(const char * szFilename, IEFileType ieft)
 		goto ReplaceDocument;
 	}
 
-	UT_Error err; 
-	err = pNewDoc->readFromFile(szFilename, ieft);
-	if (!err)
+	UT_Error errorCode; 
+	errorCode = pNewDoc->readFromFile(szFilename, ieft);
+	if (!errorCode)
 		goto ReplaceDocument;
+
+	if (createNew)
+	  {
+	    // we have a file name but couldn't load it
+	    pNewDoc->newDocument();
+
+	    // here, we want to open a new document if it doesn't exist.
+	    // errorCode could also take several other values, indicating
+	    // that the document exists but for some reason we could not
+	    // open it. in those cases, we do not wish to overwrite the
+	    // existing documents, but instead open a new blank document.
+	    // this fixes bug 1668 - DAL
+	    if ( UT_IE_FILENOTFOUND == errorCode )
+	      errorCode = pNewDoc->saveAs(szFilename, ieft);
+	  }
+	if (!errorCode)
+	  goto ReplaceDocument;
+
 	
 	UT_DEBUGMSG(("ap_Frame: could not open the file [%s]\n",szFilename));
 	UNREFP(pNewDoc);
-	return err;
+	return errorCode;
 
 ReplaceDocument:
 	getApp()->forgetClones(this);
@@ -499,20 +529,38 @@ ReplaceDocument:
 	return UT_OK;
 }
 	
-XAP_Frame * AP_BeOSFrame::cloneFrame(void)
+XAP_Frame * AP_BeOSFrame::buildFrame(XAP_Frame * pF)
 {
-	AP_BeOSFrame * pClone = new AP_BeOSFrame(this);
 	UT_Error error = UT_OK;
-	ENSUREP(pClone);
+	AP_BeOSFrame * pFrame = static_cast<AP_BeOSFrame *>(pF);
+	ENSUREP(pFrame);
+	return pFrame;
 
-	if (!pClone->initialize())
+	if (!pFrame->initialize())
 		goto Cleanup;
-	error = pClone->_showDocument();
+
+	error = pFrame->_showDocument();
 	if (error)
 		goto Cleanup;
 
-	pClone->show();
+	pFrame->show();
+	return static_cast<XAP_Frame *>(pFrame);
+								                             
+Cleanup:
+	// clean up anything we created here
+	if (pFrame)
+	{
+		m_pBeOSApp->forgetFrame(pFrame);
+		delete pFrame;
+	}
 
+	return NULL;
+}
+
+XAP_Frame * AP_BeOSFrame::cloneFrame(void)
+{
+	AP_BeOSFrame * pClone = new AP_BeOSFrame(this);
+	ENSUREP(pClone);
 	return pClone;
 
 Cleanup:
@@ -525,14 +573,13 @@ Cleanup:
 
 	return NULL;
 }
-
-UT_Error AP_BeOSFrame::loadDocument(const char * szFilename, int ieft, bool createNew)
+	
+UT_Error AP_BeOSFrame::loadDocument(const char * szFilename, int ieft)
 {
-  UT_ASSERT(UT_TODO);
-  return UT_OK;
+  return loadDocument(szFilename, ieft, false);
 }
 
-UT_Error AP_BeOSFrame::loadDocument(const char * szFilename, int ieft)
+UT_Error AP_BeOSFrame::loadDocument(const char * szFilename, int ieft, bool createNew)
 {
 	bool bUpdateClones;
 	UT_Vector vClones;
@@ -545,7 +592,7 @@ UT_Error AP_BeOSFrame::loadDocument(const char * szFilename, int ieft)
 	}
 
 	UT_Error err;
-	err = _loadDocument(szFilename, (IEFileType) ieft); 
+	err = _loadDocument(szFilename, (IEFileType) ieft, createNew); 
 	if (err)
 	{
 		// we could not load the document.
@@ -736,6 +783,19 @@ void AP_BeOSFrame::toggleBar(UT_uint32 iBarNb, bool bBarOn)
 		pToolbar->hide();
 }
 
+void AP_BeOSFrame::toggleStatusBar(bool bStatusBarOn)
+{
+        UT_DEBUGMSG(("AP_BeOSFrame::toggleStatusBar %d\n", bStatusBarOn));
+
+        AP_FrameData *pFrameData = static_cast<AP_FrameData *> (getFrameData());
+        UT_ASSERT(pFrameData);
+
+        if (bStatusBarOn)
+                pFrameData->m_pStatusBar->show();
+        else    // turning status bar off
+                pFrameData->m_pStatusBar->hide();
+}
+
 void AP_BeOSFrame::toggleRuler(bool bRulerOn)
 {
 	UT_DEBUGMSG(("AP_BeOSFrame::toggleRuler %d", bRulerOn));	
