@@ -25,7 +25,7 @@
 
 // This header defines some functions for Unix dialogs,
 // like centering them, measuring them, etc.
-#include "ut_dialogHelper.h"
+#include "xap_UnixDialogHelper.h"
 
 #include "gr_UnixGraphics.h"
 
@@ -78,7 +78,7 @@ XAP_Dialog * AP_UnixDialog_Options::static_constructor(XAP_DialogFactory * pFact
 
 AP_UnixDialog_Options::AP_UnixDialog_Options(XAP_DialogFactory * pDlgFactory,
 					     XAP_Dialog_Id id)
-  : AP_Dialog_Options(pDlgFactory,id)
+  : AP_Dialog_Options(pDlgFactory,id), m_pageSize(fp_PageSize::psLetter)
 {
 }
 
@@ -100,8 +100,8 @@ void AP_UnixDialog_Options::runModal(XAP_Frame * pFrame)
 
     // Populate the window's data items
     _populateWindowData();
-	initializeTransperentToggle(); // Hack for Transperency, sorry.
-
+	_initUnixOnlyPrefs();
+	
     // To center the dialog, we need the frame of its parent.
     XAP_UnixFrame * pUnixFrame = static_cast<XAP_UnixFrame *>(pFrame);
     UT_ASSERT(pUnixFrame);
@@ -186,6 +186,15 @@ void AP_UnixDialog_Options::event_clistClicked (int row, int col)
 
 #undef CTI
 
+static void s_page_size_changed (GtkWidget * w, GtkWidget * child, 
+				 AP_UnixDialog_Options *dlg)
+{
+  UT_ASSERT(w && dlg);
+
+  fp_PageSize::Predefined pos = (fp_PageSize::Predefined)gtk_list_child_position (GTK_LIST(w), child);
+  dlg->event_PageSizeChanged (pos);
+}
+
 void AP_UnixDialog_Options::event_ChooseTransparentColor(void)
 {
 //
@@ -281,17 +290,6 @@ void AP_UnixDialog_Options::event_AllowTransparentColor(void)
 		gtk_widget_set_sensitive(m_pushbuttonNewTransparentColor,TRUE);
 }
 
-void AP_UnixDialog_Options::initializeTransperentToggle(void)
-{
-	if(UT_strcmp(m_CurrentTransparentColor,"ffffff") == 0)
-	{
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (m_checkbuttonTransparentIsWhite), FALSE);
-	}
-	else
-	{
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (m_checkbuttonTransparentIsWhite), TRUE);
-	}
-}
 void AP_UnixDialog_Options::event_OK(void)
 {
     m_answer = AP_Dialog_Options::a_OK;
@@ -393,7 +391,6 @@ GtkWidget* AP_UnixDialog_Options::_constructWindowContents (GtkWidget * vbox)
 	GtkWidget *ruler_units;
 	GtkWidget *ruler_units_menu;
 	GtkWidget *page_size;
-	GtkWidget *page_size_menu;
 	GtkWidget *label22;
 	GtkWidget *label21;
 	GtkWidget *vbox58;
@@ -401,6 +398,8 @@ GtkWidget* AP_UnixDialog_Options::_constructWindowContents (GtkWidget * vbox)
 	GtkWidget *hbox58;
 	GtkWidget *checkWhiteForTransparent;
 	GtkWidget *pushChooseColorForTransparent;
+	GtkWidget *checkAllowCustomToolbars;
+	GtkWidget *checkAutoLoadPlugins;
 	GtkWidget *label3;
 	GtkWidget *vbox36;
 	GtkWidget *frame40;
@@ -418,7 +417,7 @@ GtkWidget* AP_UnixDialog_Options::_constructWindowContents (GtkWidget * vbox)
 	GtkWidget *autosave_time;
 	GtkWidget *frame44;
 	GtkWidget *check_splash;
-	GtkWidget *hbox29;
+	GtkWidget *vbox29;
 	GtkWidget *label23;
 	GtkWidget *hbox28;
 	GtkWidget *label24;
@@ -428,8 +427,13 @@ GtkWidget* AP_UnixDialog_Options::_constructWindowContents (GtkWidget * vbox)
 	GtkWidget *frame42;
 	GtkWidget *vbox59;
 	GtkWidget *rtl_dominant;
+	GtkWidget *use_context_glyphs;
+	GtkWidget *save_context_glyphs;
+	GtkWidget *hebrew_context_glyphs;
 #endif
-	
+	GtkWidget *fontWarning;
+	GtkWidget *fontPath;
+		
 	mainWindow = m_windowMain;
 	dialog_vbox1 = vbox;
 	gtk_widget_show (dialog_vbox1);
@@ -783,23 +787,24 @@ GtkWidget* AP_UnixDialog_Options::_constructWindowContents (GtkWidget * vbox)
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (ruler_units), ruler_units_menu);
 
 	// the page size menu
-	page_size = gtk_option_menu_new ();
+	page_size = gtk_combo_new(); 
 	gtk_widget_show (page_size);
 	gtk_table_attach (GTK_TABLE (table4), page_size, 1, 2, 1, 2,
 			  (GtkAttachOptions) (GTK_FILL),
 			  (GtkAttachOptions) (GTK_EXPAND), 0, 0);
-	page_size_menu = gtk_menu_new ();
-	for (int i = (int)fp_PageSize::A0; i < (int)fp_PageSize::Custom; i++)
+	GList *popdown_items = NULL;
+	for (int i = (int)fp_PageSize::_first_predefined_pagesize_;
+		 i < (int)fp_PageSize::_last_predefined_pagesize_dont_use_; i++)
 	{
-	    glade_menuitem = gtk_menu_item_new_with_label (fp_PageSize::PredefinedToName ((fp_PageSize::Predefined)i));
-	    gtk_object_set_data(GTK_OBJECT(glade_menuitem), WIDGET_MENU_OPTION_PTR, (gpointer) page_size);
-	    gtk_object_set_data(GTK_OBJECT(glade_menuitem), WIDGET_MENU_VALUE_TAG, GINT_TO_POINTER(i));
-	    CONNECT_MENU_ITEM_SIGNAL_ACTIVATE(glade_menuitem);
-	    gtk_widget_show (glade_menuitem);
-	    gtk_menu_append (GTK_MENU (page_size_menu), glade_menuitem);
+	  popdown_items = g_list_append (popdown_items, (void*)fp_PageSize::PredefinedToName ((fp_PageSize::Predefined)i) );
 	}
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (page_size), page_size_menu);
-	
+	gtk_combo_set_popdown_strings (GTK_COMBO (page_size), popdown_items);
+
+	GtkList * optionPageSizeList = GTK_LIST(GTK_COMBO(page_size)->list);
+	gtk_list_select_item (optionPageSizeList, (gint)m_pageSize);
+	gtk_signal_connect(GTK_OBJECT(optionPageSizeList), "select-child",
+			   GTK_SIGNAL_FUNC(s_page_size_changed), (gpointer)this);
+
 	label22 = gtk_label_new (pSS->getValue(AP_STRING_ID_DLG_Options_Label_DefaultPageSize));
 	gtk_widget_show (label22);
 	gtk_table_attach (GTK_TABLE (table4), label22, 0, 1, 1, 2,
@@ -836,8 +841,16 @@ GtkWidget* AP_UnixDialog_Options::_constructWindowContents (GtkWidget * vbox)
 	gtk_widget_show (pushChooseColorForTransparent);
 	gtk_box_pack_start (GTK_BOX (hbox58), pushChooseColorForTransparent, TRUE, TRUE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (pushChooseColorForTransparent), 2);
+//
+// Custom toolbars.
+//
+	checkAllowCustomToolbars = gtk_check_button_new_with_label (pSS->getValue(AP_STRING_ID_DLG_Options_Label_CheckAllowCustomToolbars));
+	gtk_widget_show (checkAllowCustomToolbars);
+	gtk_box_pack_start (GTK_BOX (vbox58), checkAllowCustomToolbars, TRUE, TRUE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (checkAllowCustomToolbars), 2);
+
+
 
-	
 	label3 = gtk_label_new (pSS->getValue(AP_STRING_ID_DLG_Options_Label_Layout));
 	gtk_widget_show (label3);
 	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 2), label3);
@@ -890,6 +903,21 @@ GtkWidget* AP_UnixDialog_Options::_constructWindowContents (GtkWidget * vbox)
 	rtl_dominant = gtk_check_button_new_with_label (pSS->getValue(AP_STRING_ID_DLG_Options_Label_DirectionRtl));
 	gtk_widget_show (rtl_dominant);
 	gtk_box_pack_start (GTK_BOX (vbox59), rtl_dominant, FALSE, FALSE, 0);
+	use_context_glyphs = gtk_check_button_new_with_label (pSS->getValue(AP_STRING_ID_DLG_Options_Label_UseContextGlyphs));
+	gtk_widget_show (use_context_glyphs);
+	gtk_box_pack_start (GTK_BOX (vbox59), use_context_glyphs, FALSE, FALSE, 0);
+	
+	// this is not implemented at the moment
+	save_context_glyphs = gtk_check_button_new_with_label (pSS->getValue(AP_STRING_ID_DLG_Options_Label_SaveContextGlyphs));
+#if 0
+	// currently not implemented	
+	gtk_widget_show (save_context_glyphs);
+#endif
+	gtk_box_pack_start (GTK_BOX (vbox59), save_context_glyphs, FALSE, FALSE, 0);
+	
+	hebrew_context_glyphs = gtk_check_button_new_with_label (pSS->getValue(AP_STRING_ID_DLG_Options_Label_HebrewContextGlyphs));
+	gtk_widget_show (hebrew_context_glyphs);
+	gtk_box_pack_start (GTK_BOX (vbox59), hebrew_context_glyphs, FALSE, FALSE, 0);
 #endif		
 	
 	// AUTO SAVE
@@ -937,16 +965,29 @@ GtkWidget* AP_UnixDialog_Options::_constructWindowContents (GtkWidget * vbox)
 	gtk_box_pack_start (GTK_BOX (vbox36), frame44, FALSE, TRUE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (frame44), 4);
 	
-	hbox29 = gtk_hbox_new (FALSE, 0);
-	gtk_widget_show (hbox29);
-	//gtk_box_pack_start (GTK_BOX (vbox36), hbox29, TRUE, TRUE, 0);
-	gtk_container_add (GTK_CONTAINER (frame44), hbox29);
+	vbox29 = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show (vbox29);
+	gtk_container_add (GTK_CONTAINER (frame44), vbox29);
 
 	check_splash = gtk_check_button_new_with_label (pSS->getValue(AP_STRING_ID_DLG_Options_Label_ShowSplash));
 	gtk_widget_show (check_splash);
-	gtk_box_pack_start (GTK_BOX (hbox29), check_splash, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox29), check_splash, FALSE, FALSE, 0);
 
+	fontWarning = gtk_check_button_new_with_label (pSS->getValue(XAP_STRING_ID_DLG_Options_Label_UnixFontWarning));
+	gtk_widget_show (fontWarning);
+	gtk_box_pack_start (GTK_BOX (vbox29), fontWarning, FALSE, FALSE, 0);
 
+	fontPath = gtk_check_button_new_with_label (pSS->getValue(XAP_STRING_ID_DLG_Options_Label_ModifyUnixFontPath));
+	gtk_widget_show (fontPath);
+	gtk_box_pack_start (GTK_BOX (vbox29), fontPath, FALSE, FALSE, 0);
+
+//
+// Auto Load Plugins.
+//
+	checkAutoLoadPlugins = gtk_check_button_new_with_label (pSS->getValue(AP_STRING_ID_DLG_Options_Label_CheckAutoLoadPlugins));
+	gtk_widget_show (checkAutoLoadPlugins);
+	gtk_box_pack_start (GTK_BOX (vbox29), checkAutoLoadPlugins, TRUE, TRUE, 0);
+	
 	label10 = gtk_label_new (pSS->getValue(AP_STRING_ID_DLG_Options_TabLabel_Preferences));
 	gtk_widget_show (label10);
 	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 3), label10);
@@ -957,6 +998,9 @@ GtkWidget* AP_UnixDialog_Options::_constructWindowContents (GtkWidget * vbox)
 	m_notebook = notebook1;
 #ifdef BIDI_ENABLED
 	m_checkbuttonOtherDirectionRtl = rtl_dominant;
+	m_checkbuttonOtherUseContextGlyphs = use_context_glyphs;
+	m_checkbuttonOtherSaveContextGlyphs = save_context_glyphs;
+	m_checkbuttonOtherHebrewContextGlyphs = hebrew_context_glyphs;
 #endif
 	m_checkbuttonAutoSaveFile = autosave_cb;
 	m_textAutoSaveFileExt = autosave_ext;
@@ -977,6 +1021,10 @@ GtkWidget* AP_UnixDialog_Options::_constructWindowContents (GtkWidget * vbox)
 
 	m_checkbuttonTransparentIsWhite = checkWhiteForTransparent;
 	m_pushbuttonNewTransparentColor = pushChooseColorForTransparent;
+
+	m_checkbuttonAllowCustomToolbars      = checkAllowCustomToolbars;
+	m_checkbuttonAutoLoadPlugins      = checkAutoLoadPlugins;
+
 	m_checkbuttonSmartQuotesEnable	        = enable_sq;
 	m_listDefaultPageSize			= page_size;
 
@@ -987,16 +1035,18 @@ GtkWidget* AP_UnixDialog_Options::_constructWindowContents (GtkWidget * vbox)
 		
 	m_checkbuttonViewShowRuler		= show_ruler;
 	m_listViewRulerUnits			= ruler_units;
-	m_checkbuttonViewCursorBlink	        = blink_cursor;
+	m_checkbuttonViewCursorBlink	= blink_cursor;
 
-	m_checkbuttonViewShowStatusBar	        = show_statusbar;
+	m_checkbuttonViewShowStatusBar	= show_statusbar;
 	m_checkbuttonViewAll			= view_all;
 	m_checkbuttonViewHiddenText		= view_hidden;
-	m_checkbuttonViewUnprintable	        = view_invis;
-
+	m_checkbuttonViewUnprintable	= view_invis;
+	m_checkbuttonFontWarning		= fontWarning;
+	m_checkbuttonFontPath		    = fontPath;
+	
 	// TODO: rulers
-	m_checkbuttonViewShowTB	= show_toolbar;
-	m_checkbuttonViewHideTB = hide_toolbar;
+	m_checkbuttonViewShowTB	        = show_toolbar;
+	m_checkbuttonViewHideTB         = hide_toolbar;
 	m_toolbarClist = toolbar_clist;
 
 	
@@ -1034,6 +1084,12 @@ GtkWidget* AP_UnixDialog_Options::_constructWindowContents (GtkWidget * vbox)
 			   GTK_SIGNAL_FUNC(s_chooseTransparentColor),
 			   (gpointer) this);
 
+#ifdef BIDI_ENABLED
+	gtk_signal_connect(GTK_OBJECT(m_checkbuttonOtherUseContextGlyphs),
+			   "toggled",
+			   GTK_SIGNAL_FUNC(s_checkbutton_toggle),
+			   (gpointer) this);
+#endif
 
 	_setNotebookPageNum (0);
 	//gtk_clist_select_row (GTK_CLIST (toolbar_clist), 0, 0);
@@ -1206,6 +1262,13 @@ GtkWidget *AP_UnixDialog_Options::_lookupWidget ( tControl id )
 #ifdef BIDI_ENABLED
 	case id_CHECK_OTHER_DEFAULT_DIRECTION_RTL:
 		return m_checkbuttonOtherDirectionRtl;
+		
+	case id_CHECK_OTHER_USE_CONTEXT_GLYPHS:
+		return m_checkbuttonOtherUseContextGlyphs;
+	case id_CHECK_OTHER_SAVE_CONTEXT_GLYPHS:
+		return m_checkbuttonOtherSaveContextGlyphs;
+	case id_CHECK_OTHER_HEBREW_CONTEXT_GLYPHS:
+		return m_checkbuttonOtherHebrewContextGlyphs;
 #endif
 
 	case id_CHECK_AUTO_SAVE_FILE:
@@ -1248,6 +1311,12 @@ GtkWidget *AP_UnixDialog_Options::_lookupWidget ( tControl id )
 	case id_CHECK_VIEW_UNPRINTABLE:
 		return m_checkbuttonViewUnprintable;
 
+	case id_CHECK_ALLOW_CUSTOM_TOOLBARS:
+		return m_checkbuttonAllowCustomToolbars;
+
+	case id_CHECK_AUTO_LOAD_PLUGINS:
+		return m_checkbuttonAutoLoadPlugins;
+
 	case id_CHECK_COLOR_FOR_TRANSPARENT_IS_WHITE:
 		return  m_checkbuttonTransparentIsWhite;
 
@@ -1346,6 +1415,9 @@ DEFINE_GET_SET_BOOL(SmartQuotesEnable);
 
 #ifdef BIDI_ENABLED
 DEFINE_GET_SET_BOOL(OtherDirectionRtl);
+DEFINE_GET_SET_BOOL(OtherUseContextGlyphs);
+DEFINE_GET_SET_BOOL(OtherSaveContextGlyphs);
+DEFINE_GET_SET_BOOL(OtherHebrewContextGlyphs);
 #endif
 
 DEFINE_GET_SET_BOOL(AutoSaveFile);
@@ -1391,12 +1463,6 @@ UT_Dimension AP_UnixDialog_Options::_gatherViewRulerUnits(void)
 	return (UT_Dimension)((gint)gtk_object_get_data( GTK_OBJECT(m_listViewRulerUnits), WIDGET_MENU_VALUE_TAG )); 
 }			
 
-fp_PageSize::Predefined AP_UnixDialog_Options::_gatherDefaultPageSize(void)
-{
-	UT_ASSERT(m_listDefaultPageSize && GTK_IS_OPTION_MENU(m_listDefaultPageSize));
-	return (fp_PageSize::Predefined) ((gint)gtk_object_get_data( GTK_OBJECT(m_listDefaultPageSize), WIDGET_MENU_VALUE_TAG ));
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 // This function will lookup a option box by the value stored in the 
 //	user data under the key WIDGET_MENU_VALUE_TAG
@@ -1426,7 +1492,7 @@ static void search_for_value ( GtkWidget *widget, gpointer _value )
 }
 
 // returns -1 if not found
-int option_menu_set_by_key ( GtkWidget *option_menu, gpointer value, gchar *key )
+static int option_menu_set_by_key ( GtkWidget *option_menu, gpointer value, gchar *key )
 {
 	UT_ASSERT( option_menu && key && GTK_IS_OPTION_MENU(option_menu));
 
@@ -1456,20 +1522,26 @@ int option_menu_set_by_key ( GtkWidget *option_menu, gpointer value, gchar *key
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
+
+fp_PageSize::Predefined AP_UnixDialog_Options::_gatherDefaultPageSize(void)
+{
+	return (fp_PageSize::Predefined)m_pageSize;
+}
+
+void AP_UnixDialog_Options::_setDefaultPageSize(fp_PageSize::Predefined pre)
+{
+	UT_ASSERT(m_listDefaultPageSize);
+	m_pageSize = pre;
+	GtkList * optionPageSizeList = GTK_LIST(GTK_COMBO(m_listDefaultPageSize)->list);
+	gtk_list_select_item (optionPageSizeList, (gint)pre);
+}
+
 void    AP_UnixDialog_Options::_setViewRulerUnits(UT_Dimension dim) 
 {	
 	UT_ASSERT(m_listViewRulerUnits && GTK_IS_OPTION_MENU(m_listViewRulerUnits)); 
 
 	int r = option_menu_set_by_key ( m_listViewRulerUnits, (gpointer)dim, WIDGET_MENU_VALUE_TAG ); 
-	UT_ASSERT( r != -1 );
-}
-
-void AP_UnixDialog_Options::_setDefaultPageSize(fp_PageSize::Predefined pre)
-{
-	UT_ASSERT(m_listDefaultPageSize && GTK_IS_OPTION_MENU(m_listDefaultPageSize));
-
-	int r = option_menu_set_by_key ( m_listDefaultPageSize, (gpointer)pre, WIDGET_MENU_VALUE_TAG );
-	UT_ASSERT( r != -1 );
 }
 
 DEFINE_GET_SET_BOOL	(ViewCursorBlink);
@@ -1477,6 +1549,8 @@ DEFINE_GET_SET_BOOL	(ViewCursorBlink);
 DEFINE_GET_SET_BOOL	(ViewAll);
 DEFINE_GET_SET_BOOL	(ViewHiddenText);
 DEFINE_GET_SET_BOOL	(ViewUnprintable);
+DEFINE_GET_SET_BOOL (AllowCustomToolbars);
+DEFINE_GET_SET_BOOL (AutoLoadPlugins);
 
 #undef DEFINE_GET_SET_BOOL
 
@@ -1579,6 +1653,7 @@ void    AP_UnixDialog_Options::_setNotebookPageNum(int pn)
 	UT_ASSERT(w && GTK_IS_WIDGET(w));
 
 	int i = (int) gtk_object_get_data( GTK_OBJECT(w), "tControl" );
+	UT_DEBUGMSG(("s_checkbutton_toggle: control id = %d\n", i));
 	dlg->_enableDisableLogic( (AP_Dialog_Options::tControl) i );
 }
 
@@ -1610,3 +1685,47 @@ void    AP_UnixDialog_Options::_setNotebookPageNum(int pn)
   AP_UnixDialog_Options * dlg = static_cast <AP_UnixDialog_Options *>(d);
   dlg->event_clistClicked (row, col);
 }
+
+void AP_UnixDialog_Options::_initUnixOnlyPrefs()
+{
+	if(UT_strcmp(m_CurrentTransparentColor,"ffffff") == 0)
+	{
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (m_checkbuttonTransparentIsWhite), FALSE);
+	}
+	else
+	{
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (m_checkbuttonTransparentIsWhite), TRUE);
+	}
+	
+	bool bFontWarning;
+	bool bRet = m_pApp->getPrefsValueBool(XAP_PREF_KEY_ShowUnixFontWarning, &bFontWarning);
+    UT_DEBUGMSG(("bRet %d, bFontWarning %d\n",bRet,bFontWarning));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(m_checkbuttonFontWarning), bFontWarning);
+
+	bRet = m_pApp->getPrefsValueBool(XAP_PREF_KEY_ModifyUnixFontPath, &bFontWarning);
+    UT_DEBUGMSG(("bRet %d, FontPath %d\n",bRet,bFontWarning));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(m_checkbuttonFontPath), bFontWarning);
+}
+
+void AP_UnixDialog_Options::_saveUnixOnlyPrefs()
+{
+	XAP_Prefs *pPrefs = m_pApp->getPrefs();
+	UT_ASSERT(pPrefs);
+
+	XAP_PrefsScheme *pPrefsScheme = pPrefs->getCurrentScheme();
+	UT_ASSERT(pPrefsScheme);
+	
+	bool bRet = pPrefsScheme->setValueBool(XAP_PREF_KEY_ShowUnixFontWarning,
+			       gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_checkbuttonFontWarning)));
+	UT_DEBUGMSG(("bRet %d, Font warning %d\n",bRet,gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_checkbuttonFontWarning))));
+	
+	bRet = pPrefsScheme->setValueBool(XAP_PREF_KEY_ModifyUnixFontPath,
+			       gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_checkbuttonFontPath)));
+	UT_DEBUGMSG(("bRet %d, Font path %d\n",bRet,gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_checkbuttonFontPath))));
+}
+
+void AP_UnixDialog_Options::_storeWindowData(void)
+{
+	_saveUnixOnlyPrefs();
+	AP_Dialog_Options::_storeWindowData();
+}
