@@ -88,7 +88,7 @@
 #if 0
 #include "ap_NautilusView.h"
 #include <libnautilus/nautilus-view-standard-main.h>
-#endif 0
+#endif
 
 
 // quick hack - this is defined in ap_EditMethods.cpp
@@ -255,7 +255,7 @@ int AP_UnixGnomeApp::main(const char * szAppName, int argc, char ** argv)
 	// not running as a bonobo app
 
 #ifndef ABI_OPT_WIDGET
-	gnome_init_with_popt_table("AbiWord", "1.0.0", argc, argv, options, 0, &poptcon);
+	gnome_init_with_popt_table("AbiWord", "1.0.5", argc, argv, options, 0, &poptcon);
 #endif
 
 	// hack needed to intialize gtk before ::initialize
@@ -352,9 +352,8 @@ int AP_UnixGnomeApp::main(const char * szAppName, int argc, char ** argv)
 		XParseGeometry(geometry, &x, &y, &width, &height);
 		
 		// use both by default
-		XAP_UNIXBASEAPP::windowGeometryFlags f = (XAP_UNIXBASEAPP::windowGeometryFlags)
-			(XAP_UNIXBASEAPP::GEOMETRY_FLAG_SIZE
-			 | XAP_UNIXBASEAPP::GEOMETRY_FLAG_POS);
+		UT_uint32 f = (XAP_UNIXBASEAPP::GEOMETRY_FLAG_SIZE
+		               | XAP_UNIXBASEAPP::GEOMETRY_FLAG_POS);
 		
 		// if pos (x and y) weren't provided just use size
 		if (x == dummy || y == dummy)
@@ -548,7 +547,7 @@ bool AP_UnixGnomeApp::parseCommandLine(poptContext poptcon)
 //-------------------------------------------------------------------
 
 /*****************************************************************/
-/* Implements the Bonobo/PropertyBag:1.0 interface
+/* Implements the Bonobo/PropertyBag:1.0 interface               */
 /*****************************************************************/
 
 /* 
@@ -934,7 +933,7 @@ print_document (GnomePrintContext         *ctx,
   // actually print
   s_actuallyPrint ( pDoc, pGraphics,
 		    pPrintView, "bonobo_printed_document",
-		    1, false,
+		    1, false, false,
 		    width, height,
 		    1, iPagesToPrint ) ;
   
