@@ -199,10 +199,8 @@ UT_Error AP_Win32Frame::_showDocument(UT_uint32 iZoom)
 	pView = new FV_View(getApp(), this, pDocLayout);
 	ENSUREP(pView);
 
-	if (m_pView != NULL)
-	{
-		point = ((FV_View *) m_pView)->getPoint();
-	}
+//	pDocLayout->fillLayouts();
+
 
 	// The "AV_ScrollObj pScrollObj" receives
 	// send{Vertical,Horizontal}ScrollEvents
@@ -315,13 +313,35 @@ UT_Error AP_Win32Frame::_showDocument(UT_uint32 iZoom)
 	m_pView->setWindowSize(r.right - r.left, r.bottom - r.top);
 	InvalidateRect(hwnd, NULL, TRUE);
 
+	updateTitle();
+
+	pDocLayout->fillLayouts();
+
+	// we cannot do this before we fill the layout, because if we are
+	// running in default RTL mode, the X scroll needs to be able to
+	// have a valid block for its calculations
 	setXScrollRange();
 	setYScrollRange();
 
-	updateTitle();
-
+	if (m_pView != NULL)
+	{
+		// we cannot just set the insertion position to that of the previous
+		// view, since the new document could be shorter or completely
+		// different from the previous one (see bug 2615)
+		// Instead we have to test that the original position is within
+		// the editable bounds, and if not, we will set the point
+		// to the end of the document (i.e., if reloading an earlier
+		// version of the same document we try to get the point as near
+		// the users editing position as possible
+		point = ((FV_View *) m_pView)->getPoint();
+		PT_DocPosition posEOD;
+		static_cast<FV_View *>(pView)->getEditableBounds(true, posEOD, false);
+		if(point > posEOD)
+			point = posEOD;
+	}
 	if (point != 0)
 		((FV_View *) m_pView)->moveInsPtTo(point);
+	m_pView->draw();
 
 	if (static_cast<AP_FrameData*>(m_pData)->m_pTopRuler)
 		static_cast<AP_FrameData*>(m_pData)->m_pTopRuler->draw(NULL);
@@ -470,6 +490,7 @@ AP_Win32Frame::AP_Win32Frame(AP_Win32Frame * f)
 	m_bMouseActivateReceived(false),
 	m_hWndHScroll(0),
 	m_hWndVScroll(0),
+	m_bFirstAfterFocus(false),
 	m_hWndGripperHack(0)
 {
 	m_hwndContainer = NULL;
@@ -507,7 +528,11 @@ XAP_Frame* AP_Win32Frame::cloneFrame(void)
 	AP_Win32Frame* pClone = new AP_Win32Frame(this);
 
 	UT_ASSERT(pClone);
-
+	return pClone;
+}
+XAP_Frame * AP_Win32Frame::buildFrame(XAP_Frame * pF)
+{
+	AP_Win32Frame * pClone = static_cast<AP_Win32Frame *>(pF);
 	if (pClone && pClone->initialize())
 	{
 		const UT_Error error = pClone->_showDocument();
@@ -621,6 +646,12 @@ void AP_Win32Frame::_getRulerSizes(int &yTopRulerHeight, int &xLeftRulerWidth)
 }
 
 void AP_Win32Frame::_createRulers(void)
+{
+	_createTopRuler();
+	_createLeftRuler();
+}
+
+void AP_Win32Frame::_createTopRuler(void)
 {
 	RECT r;
 	int cxVScroll, cyHScroll;
@@ -630,26 +661,57 @@ void AP_Win32Frame::_createRulers(void)
 	cyHScroll = GetSystemMetrics(SM_CYHSCROLL);
 
 	// create the top ruler
-
 	AP_Win32TopRuler * pWin32TopRuler = new AP_Win32TopRuler(this);
 	UT_ASSERT(pWin32TopRuler);
 	m_hwndTopRuler = pWin32TopRuler->createWindow(m_hwndContainer,
 												  0,0, (r.right - cxVScroll));
 	static_cast<AP_FrameData*>(m_pData)->m_pTopRuler = pWin32TopRuler;
-	UT_uint32 yTopRulerHeight = pWin32TopRuler->getHeight();
+
+
+	// get the width from the left ruler and stuff it into the top ruler.
+	UT_uint32 xLeftRulerWidth = 0;
+    if( m_hwndLeftRuler )
+	{
+		AP_Win32LeftRuler * pWin32LeftRuler = NULL;
+		pWin32LeftRuler =  (AP_Win32LeftRuler *) static_cast<AP_FrameData*>(m_pData)->m_pLeftRuler;
+		xLeftRulerWidth = pWin32LeftRuler->getWidth();
+	}
+	pWin32TopRuler->setOffsetLeftRuler(xLeftRulerWidth);
+}
+
+void AP_Win32Frame::_createLeftRuler(void)
+{
+	RECT r;
+	int cxVScroll, cyHScroll;
+
+	GetClientRect(m_hwndContainer,&r);
+	cxVScroll = GetSystemMetrics(SM_CXVSCROLL);
+	cyHScroll = GetSystemMetrics(SM_CYHSCROLL);
+
+	UT_uint32 yTopRulerHeight = 0;
+
+	if( m_hwndTopRuler )
+	{
+		AP_Win32TopRuler * pWin32TopRuler = NULL;
+		pWin32TopRuler =  (AP_Win32TopRuler * ) static_cast<AP_FrameData*>(m_pData)->m_pTopRuler;
+		yTopRulerHeight = pWin32TopRuler->getHeight();
+	}
 
 	// create the left ruler
-
 	AP_Win32LeftRuler * pWin32LeftRuler = new AP_Win32LeftRuler(this);
 	UT_ASSERT(pWin32LeftRuler);
 	m_hwndLeftRuler = pWin32LeftRuler->createWindow(m_hwndContainer,0,yTopRulerHeight,
 													r.bottom - yTopRulerHeight - cyHScroll);
 	static_cast<AP_FrameData*>(m_pData)->m_pLeftRuler = pWin32LeftRuler;
-	UT_uint32 xLeftRulerWidth = pWin32LeftRuler->getWidth();
 
 	// get the width from the left ruler and stuff it into the top ruler.
-
-	pWin32TopRuler->setOffsetLeftRuler(xLeftRulerWidth);
+    if( m_hwndTopRuler )
+	{
+		UT_uint32 xLeftRulerWidth = pWin32LeftRuler->getWidth();
+		AP_Win32TopRuler * pWin32TopRuler = NULL;
+		pWin32TopRuler =  (AP_Win32TopRuler * ) static_cast<AP_FrameData*>(m_pData)->m_pTopRuler;
+		pWin32TopRuler->setOffsetLeftRuler(xLeftRulerWidth);
+	}
 }
 
 UT_Error AP_Win32Frame::loadDocument(const char * szFilename, int ieft, bool createNew)
@@ -964,30 +1026,43 @@ LRESULT CALLBACK AP_Win32Frame::_ContainerWndProc(HWND hwnd, UINT iMsg, WPARAM w
  	{
  		// Get delta
  		const int iDelta = (short) HIWORD(wParam);
+		const int cWheelLines = GetMouseWheelLines();
 
- 		// Calculate the movement offset to an integer resolution
- 		const int iMove = (iDelta * GetMouseWheelLines()) / WHEEL_DELTA;
+		if (WHEEL_PAGESCROLL == cWheelLines)
+		{
+			WORD wDir = (iDelta < 0) ? SB_PAGEDOWN : SB_PAGEUP;
+			SendMessage(hwnd,
+						WM_VSCROLL,
+						MAKELONG(wDir, 0),
+						NULL);
+		}
+		else
+		{
+			// Calculate the movement offset to an integer resolution
+			const int iMove = (iDelta * cWheelLines) / WHEEL_DELTA;
 
- 		// Get current scroll position
- 		SCROLLINFO si = { 0 };
+			// Get current scroll position
+			SCROLLINFO si = { 0 };
 
- 		si.cbSize = sizeof(si);
- 		si.fMask = SIF_ALL;
- 		f->_getVerticalScrollInfo(&si);
+			si.cbSize = sizeof(si);
+			si.fMask = SIF_ALL;
+			f->_getVerticalScrollInfo(&si);
 
- 		// Clip new position to limits
- 		int iNewPos = si.nPos - (iMove * SCROLL_LINE_SIZE);
- 		if (iNewPos > si.nMax) iNewPos = si.nMax;
- 		if (iNewPos < si.nMin) iNewPos = si.nMin;
+			// Clip new position to limits
+			int iNewPos = si.nPos - (iMove * SCROLL_LINE_SIZE);
+			if (iNewPos > si.nMax) iNewPos = si.nMax;
+			if (iNewPos < si.nMin) iNewPos = si.nMin;
 
- 		if (iNewPos != si.nPos)
- 		{
- 			// If position has changed set new position
- 			SendMessage(hwnd,
- 						WM_VSCROLL,
- 						MAKELONG(SB_THUMBPOSITION, iNewPos),
- 						NULL);
- 		}
+			if (iNewPos != si.nPos)
+			{
+				// If position has changed set new position
+				iNewPos >>= f->m_vScale;
+				SendMessage(hwnd,
+							WM_VSCROLL,
+							MAKELONG(SB_THUMBPOSITION, iNewPos),
+							NULL);
+			}
+		}
 	}
 	return 0;
 
@@ -1025,6 +1100,8 @@ void AP_Win32Frame::_onSize(int nWidth, int nHeight)
 			   nHeight - yTopRulerHeight - cyHScroll, TRUE);
 }
 
+
+
 LRESULT CALLBACK AP_Win32Frame::_DocumentWndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
 {
 	AP_Win32Frame * f = GWL(hwnd);
@@ -1043,13 +1120,18 @@ LRESULT CALLBACK AP_Win32Frame::_DocumentWndProc(HWND hwnd, UINT iMsg, WPARAM wP
 		f->m_bMouseActivateReceived = true;
 		return MA_ACTIVATE;
 
-	case WM_SETFOCUS:
+	case WM_SETFOCUS:			
 		if (pView)
 		{
 			pView->focusChange(AV_FOCUS_HERE);
+			
+			if (GetKeyState(VK_LBUTTON)>0)
+				f->m_bFirstAfterFocus = true;
 
 			if(!f->m_bMouseActivateReceived)
 			{
+				
+				
 				// HACK:	Capture leaving a tool bar combo.
 				// We need to get a mouse down signal.
 				// windows is not activated so send a mouse down if the mouse is pressed.
@@ -1064,7 +1146,7 @@ LRESULT CALLBACK AP_Win32Frame::_DocumentWndProc(HWND hwnd, UINT iMsg, WPARAM wP
 	case WM_KILLFOCUS:
 		if (pView)
 		{
-			pView->focusChange(AV_FOCUS_NONE);
+			pView->focusChange(AV_FOCUS_NONE);			
 		}
 		return 0;
 
@@ -1081,14 +1163,22 @@ LRESULT CALLBACK AP_Win32Frame::_DocumentWndProc(HWND hwnd, UINT iMsg, WPARAM wP
 		return 1;
 
 	case WM_LBUTTONDOWN:
-		if(GetFocus() != hwnd)
+		
+		/* When we get the focus back 					*/
+		if (f->m_bFirstAfterFocus)
+			f->m_bFirstAfterFocus = false;	
+		else
 		{
-			SetFocus(hwnd);
-			pView = f->m_pView;
-			pMouse = (EV_Win32Mouse *) f->m_pMouse;
+			
+			if(GetFocus() != hwnd)
+			{
+				SetFocus(hwnd);
+				pView = f->m_pView;
+				pMouse = (EV_Win32Mouse *) f->m_pMouse;
+			}
+			pMouse->onButtonDown(pView,hwnd,EV_EMB_BUTTON1,wParam,LOWORD(lParam),HIWORD(lParam));
+			f->m_bMouseActivateReceived = false;
 		}
-		pMouse->onButtonDown(pView,hwnd,EV_EMB_BUTTON1,wParam,LOWORD(lParam),HIWORD(lParam));
-		f->m_bMouseActivateReceived = false;
 		return 0;
 	case WM_MBUTTONDOWN:
 		f->_startTracking(LOWORD(lParam), HIWORD(lParam));
@@ -1111,17 +1201,23 @@ LRESULT CALLBACK AP_Win32Frame::_DocumentWndProc(HWND hwnd, UINT iMsg, WPARAM wP
 	case WM_SYSKEYDOWN:
 	case WM_KEYDOWN:
 	{
-	    ev_Win32Keyboard *pWin32Keyboard = static_cast<ev_Win32Keyboard *>(f->m_pKeyboard);
-		if (pWin32Keyboard->onKeyDown(pView,hwnd,iMsg,wParam,lParam))
-			return 0;
-		return DefWindowProc(hwnd,iMsg,wParam,lParam);
+		UT_DEBUGMSG(("WM_KEYDOWN %d  - %d\n",wParam, lParam));
+	    ev_Win32Keyboard *pWin32Keyboard = static_cast<ev_Win32Keyboard *>(f->m_pKeyboard);	    
+	    
+     	if (pWin32Keyboard->onKeyDown(pView,hwnd,iMsg,wParam,lParam))
+     		return DefWindowProc(hwnd,iMsg,wParam,lParam);
+ 		else
+ 			return false;	    				 	     			
+    		
 	}
 	case WM_SYSCHAR:
 	case WM_CHAR:
 	{
+		UT_DEBUGMSG(("WM_CHAR %d  - %d\n",wParam, lParam));
 	    ev_Win32Keyboard *pWin32Keyboard = static_cast<ev_Win32Keyboard *>(f->m_pKeyboard);
-		if (pWin32Keyboard->onChar(pView,hwnd,iMsg,wParam,lParam))
-			return 0;
+	   
+	    
+		pWin32Keyboard->onChar(pView,hwnd,iMsg,wParam,lParam);		
 		return DefWindowProc(hwnd,iMsg,wParam,lParam);
 	}
 	case WM_IME_CHAR:
@@ -1314,6 +1410,12 @@ UT_Error AP_Win32Frame::_replaceDocument(AD_Document * pDoc)
 }
 
 void AP_Win32Frame::toggleRuler(bool bRulerOn)
+{
+	toggleTopRuler( bRulerOn );
+	toggleLeftRuler( bRulerOn );
+}
+
+void AP_Win32Frame::toggleTopRuler(bool bRulerOn)
 {
 	AP_FrameData *pFrameData = static_cast<AP_FrameData*>(getFrameData());
 	UT_ASSERT(pFrameData);
@@ -1321,12 +1423,10 @@ void AP_Win32Frame::toggleRuler(bool bRulerOn)
 	if (bRulerOn)
 	{
 		UT_ASSERT(!pFrameData->m_pTopRuler);
-		UT_ASSERT(!pFrameData->m_pLeftRuler);
 
-		_createRulers();
+		_createTopRuler();
 
 		static_cast<AP_FrameData*>(m_pData)->m_pTopRuler->setView(m_pView, getZoomPercentage());
-		static_cast<AP_FrameData*>(m_pData)->m_pLeftRuler->setView(m_pView, getZoomPercentage());
 	}
 	else
 	{
@@ -1334,14 +1434,9 @@ void AP_Win32Frame::toggleRuler(bool bRulerOn)
 		if (m_hwndTopRuler)
 			DestroyWindow(m_hwndTopRuler);
 
-		if (m_hwndLeftRuler)
-			DestroyWindow(m_hwndLeftRuler);
-
 		DELETEP(static_cast<AP_FrameData*>(m_pData)->m_pTopRuler);
-		DELETEP(static_cast<AP_FrameData*>(m_pData)->m_pLeftRuler);
 
 		m_hwndTopRuler = NULL;
-		m_hwndLeftRuler = NULL;
 	}
 
 	// repack the child windows
@@ -1350,6 +1445,42 @@ void AP_Win32Frame::toggleRuler(bool bRulerOn)
 	_onSize(r.right - r.left, r.bottom - r.top);
 }
 
+void AP_Win32Frame::toggleLeftRuler(bool bRulerOn)
+{
+	AP_FrameData *pFrameData = static_cast<AP_FrameData*>(getFrameData());
+	UT_ASSERT(pFrameData);
+
+	if (bRulerOn)
+	{
+		//
+		// If There is an old ruler just return
+		//
+		if(m_hwndLeftRuler)
+		{
+			return;
+		}
+		UT_ASSERT(!pFrameData->m_pLeftRuler);
+
+		_createLeftRuler();
+
+		static_cast<AP_FrameData*>(m_pData)->m_pLeftRuler->setView(m_pView, getZoomPercentage());
+	}
+	else
+	{
+		// delete the actual widgets
+		if (m_hwndLeftRuler)
+			DestroyWindow(m_hwndLeftRuler);
+
+		DELETEP(static_cast<AP_FrameData*>(m_pData)->m_pLeftRuler);
+
+		m_hwndLeftRuler = NULL;
+	}
+
+	// repack the child windows
+	RECT r;
+	GetClientRect(m_hwndContainer, &r);
+	_onSize(r.right - r.left, r.bottom - r.top);
+}
 
 /////////////////////////////////////////////////////////////////////////
 
@@ -1488,6 +1619,8 @@ void AP_Win32Frame::toggleStatusBar(bool bStatusBarOn)
 	{
 		pFrameData->m_pStatusBar->hide();
 	}
+
+	UpdateWindow(m_hwndContainer);
 }
 
 void AP_Win32Frame::_showOrHideToolbars(void)
