@@ -37,6 +37,9 @@
 #include "fv_View.h"
 #include "ap_FrameData.h"
 #include "ap_Prefs.h"
+#include "pd_Document.h"
+#include "ut_Script.h"
+#include "spell_manager.h"
 
 #define ABIWORD_VIEW  	FV_View * pView = static_cast<FV_View *>(pAV_View)
 
@@ -387,6 +390,54 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_WindowMore)
 /*****************************************************************/
 /*****************************************************************/
 
+Defun_EV_GetMenuItemState_Fn(ap_GetState_Spelling)
+{
+  EV_Menu_ItemState s = EV_MIS_ZERO ;
+
+  XAP_Prefs *pPrefs = XAP_App::getApp()->getPrefs();
+  UT_ASSERT( pPrefs );
+
+  bool b = true ;
+  pPrefs->getPrefsValueBool((XML_Char*)AP_PREF_KEY_AutoSpellCheck,&b) ;
+
+  // if there are no loaded dictionaries and we are spell checking
+  // as we type
+ if ( SpellManager::instance ().numLoadedDicts() == 0 && b )
+   s = EV_MIS_Gray;
+
+ // either have a loaded dictionary or want to spell-check manually. allow
+
+ return s;
+}
+
+Defun_EV_GetMenuItemState_Fn(ap_GetState_ColumnsActive)
+{
+  ABIWORD_VIEW ;
+  UT_ASSERT(pView) ;
+
+  EV_Menu_ItemState s = EV_MIS_ZERO ;
+
+  if(pView->isHdrFtrEdit())
+    s = EV_MIS_Gray;
+
+  return s;
+}
+
+Defun_EV_GetMenuItemState_Fn(ap_GetState_SomethingSelected)
+{
+	ABIWORD_VIEW ;
+	UT_ASSERT(pView) ;
+
+	EV_Menu_ItemState s = EV_MIS_ZERO ;
+
+	if ( pView->isSelectionEmpty () )
+	  {
+	    s = EV_MIS_Gray ;
+	  }
+
+	return s ;
+}
+
 Defun_EV_GetMenuItemState_Fn(ap_GetState_Suggest)
 {
 	ABIWORD_VIEW;
@@ -480,8 +531,19 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Changes)
 
 	EV_Menu_ItemState s = EV_MIS_ZERO;
 
+	XAP_Frame * pFrame = static_cast<XAP_Frame *> (pAV_View->getParentData());
+	UT_ASSERT(pFrame);
+
+	AP_FrameData *pFrameData = static_cast<AP_FrameData *> (pFrame->getFrameData());
+	UT_ASSERT(pFrameData);
+
 	switch(id)
 	{
+	case AP_MENU_ID_FILE_REVERT:
+	case AP_MENU_ID_FILE_SAVE:
+	  if (!pView->getDocument()->isDirty() || !pView->canDo(true))
+	    s = EV_MIS_Gray;
+	  break;
 	case AP_MENU_ID_EDIT_UNDO:
 		if (!pView->canDo(true))
 			s = EV_MIS_Gray;
@@ -492,6 +554,16 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Changes)
 			s = EV_MIS_Gray;
 		break;
 
+	case AP_MENU_ID_EDIT_EDITHEADER:
+//	  if (pFrameData->m_pViewMode == VIEW_NORMAL)
+//	    s = EV_MIS_Gray;
+	  break;
+
+	case AP_MENU_ID_EDIT_EDITFOOTER:
+//	  if (pFrameData->m_pViewMode == VIEW_NORMAL)
+//	    s = EV_MIS_Gray;
+	  break;
+
 	case AP_MENU_ID_EDIT_REMOVEHEADER:
 		if (!pView->isHeaderOnPage())
 			s = EV_MIS_Gray;
@@ -510,6 +582,19 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Changes)
 	return s;
 }
 
+Defun_EV_GetMenuItemState_Fn(ap_GetState_ScriptsActive)
+{
+  EV_Menu_ItemState s = EV_MIS_ZERO;
+
+  UT_ScriptLibrary& instance = UT_ScriptLibrary::instance ();
+  UT_uint32 filterCount = instance.getNumScripts ();
+
+  if ( filterCount == 0 )
+    s = EV_MIS_Gray;
+
+  return s;
+}
+
 Defun_EV_GetMenuItemState_Fn(ap_GetState_Selection)
 {
         ABIWORD_VIEW;
@@ -601,6 +686,10 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_CharFmt)
 	const XML_Char * prop = NULL;
 	const XML_Char * val  = NULL;
 
+	if(pView->getDocument()->areStylesLocked()) {
+          return EV_MIS_Gray;
+	}
+
 	switch(id)
 	{
 	case AP_MENU_ID_FMT_BOLD:
@@ -685,6 +774,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_CharFmt)
 		free(props_in);
 	}
 
+
 	return s;
 }
 
@@ -698,6 +788,10 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_BlockFmt)
 	const XML_Char * prop = "text-align";
 	const XML_Char * val  = NULL;
 
+	if(pView->getDocument()->areStylesLocked()) {
+	    return EV_MIS_Gray;
+	}
+
 	switch(id)
 	{
 	case AP_MENU_ID_ALIGN_LEFT:
@@ -833,3 +927,15 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_View)
 
 	return s;
 }
+
+Defun_EV_GetMenuItemState_Fn(ap_GetState_StylesLocked)
+{
+	ABIWORD_VIEW;
+	UT_ASSERT(pView);
+
+        if(pView->getDocument()->areStylesLocked()) {
+            return EV_MIS_Gray;
+        }
+
+        return EV_MIS_ZERO;
+}
