@@ -34,10 +34,33 @@
 #include "xap_EncodingManager.h"
 #include "ut_misc.h"
 
+/*****************************************************************/	
+/*****************************************************************/	
+
+#define X_TestParseState(ps)	((m_parseState==(ps)))
+
+#define X_VerifyParseState(ps)	do {  if (!(X_TestParseState(ps)))			\
+									  {  m_error = UT_IE_BOGUSDOCUMENT;	\
+										 return; } } while (0)
+
+#define X_CheckDocument(b)		do {  if (!(b))								\
+									  {  m_error = UT_IE_BOGUSDOCUMENT;	\
+										 return; } } while (0)
+
+#define X_CheckError(v)			do {  if (!(v))								\
+									  {  m_error = UT_ERROR;			\
+										 return; } } while (0)
+
+#define	X_EatIfAlreadyError()	do {  if (m_error) return; } while (0)
+
+#define X_VerifyInsideBlockOrField()  do { if(!(X_TestParseState(_PS_Block) || X_TestParseState(_PS_Field))) \
+                                                                          {  m_error = UT_IE_BOGUSDOCUMENT;  \
+                                                                             return; } } while (0)
+
 /*****************************************************************/
 /*****************************************************************/
 
-bool IE_Imp_AbiWord_1_Sniffer::recognizeContents (const char * szBuf, 
+UT_Confidence_t IE_Imp_AbiWord_1_Sniffer::recognizeContents (const char * szBuf, 
 												  UT_uint32 iNumbytes)
 {
 	UT_uint32 iLinesToRead = 6 ;  // Only examine the first few lines of the file
@@ -48,16 +71,19 @@ bool IE_Imp_AbiWord_1_Sniffer::recognizeContents (const char * szBuf,
 	while( iLinesToRead-- )
 	{
 		magic = "<abiword " ;
-		if ( (iNumbytes - iBytesScanned) < strlen(magic) ) return(false);
-		if ( strncmp(p, magic, strlen(magic)) == 0 ) return(true);
+		if ( (iNumbytes - iBytesScanned) < strlen(magic) ) return(UT_CONFIDENCE_ZILCH);
+		if ( strncmp(p, magic, strlen(magic)) == 0 ) return(UT_CONFIDENCE_PERFECT);
+	       magic = "<awml " ;
+	       if ( (iNumbytes - iBytesScanned) < strlen(magic) ) return(UT_CONFIDENCE_ZILCH);
+		if ( strncmp(p, magic, strlen(magic)) == 0 ) return(UT_CONFIDENCE_PERFECT);
 		magic = "<!-- This file is an AbiWord document." ;
-		if ( (iNumbytes - iBytesScanned) < strlen(magic) ) return(false);
-		if ( strncmp(p, magic, strlen(magic)) == 0 ) return(true);
+		if ( (iNumbytes - iBytesScanned) < strlen(magic) ) return(UT_CONFIDENCE_ZILCH);
+		if ( strncmp(p, magic, strlen(magic)) == 0 ) return(UT_CONFIDENCE_PERFECT);
 		/*  Seek to the next newline:  */
 		while ( *p != '\n' && *p != '\r' )
 		{
 			iBytesScanned++ ; p++ ;
-			if( iBytesScanned+2 >= iNumbytes ) return(false);
+			if( iBytesScanned+2 >= iNumbytes ) return(UT_CONFIDENCE_ZILCH);
 		}
 		/*  Seek past the next newline:  */
 		if ( *p == '\n' || *p == '\r' )
@@ -69,12 +95,14 @@ bool IE_Imp_AbiWord_1_Sniffer::recognizeContents (const char * szBuf,
 			}
 		}
 	}
-	return(false);
+	return UT_CONFIDENCE_ZILCH;
 }
 
-bool IE_Imp_AbiWord_1_Sniffer::recognizeSuffix (const char * szSuffix)
+UT_Confidence_t IE_Imp_AbiWord_1_Sniffer::recognizeSuffix (const char * szSuffix)
 {
-	return (!UT_stricmp(szSuffix, ".abw") || !UT_stricmp(szSuffix, ".awt"));
+  if (!UT_stricmp(szSuffix, ".abw") || !UT_stricmp(szSuffix, ".awt"))
+    return UT_CONFIDENCE_PERFECT;
+  return UT_CONFIDENCE_ZILCH;
 }
 
 bool IE_Imp_AbiWord_1_Sniffer::getDlgLabels (const char ** szDesc,
@@ -100,20 +128,23 @@ UT_Error IE_Imp_AbiWord_1_Sniffer::constructImporter (PD_Document * pDocument,
 
 IE_Imp_AbiWord_1::~IE_Imp_AbiWord_1()
 {
+  if ( !m_bWroteSection )
+    X_CheckError(getDoc()->appendStrux(PTX_Section,NULL));
+  if ( !m_bWroteParagraph )
+    X_CheckError(getDoc()->appendStrux(PTX_Block,NULL));
 }
 
 IE_Imp_AbiWord_1::IE_Imp_AbiWord_1(PD_Document * pDocument)
-  : IE_Imp_XML(pDocument, true)
+  : IE_Imp_XML(pDocument, true), m_bWroteSection (false),
+    m_bWroteParagraph(false), m_bDocHasLists(false), m_bDocHasPageSize(false)
 {
-	m_bDocHasLists = false;
-	m_bDocHasPageSize = false;
 }
 
 /* Quick hack for GZipAbiWord */
 UT_Error IE_Imp_AbiWord_1::importFile(const char * szFilename)
 {
 	UT_Error bret = IE_Imp_XML::importFile(szFilename);
-	if(m_bDocHasPageSize == false)
+	if(!bret && !m_bDocHasPageSize)
 		getDoc()->setDefaultPageSize();
 	return bret;
 }
@@ -141,6 +172,8 @@ UT_Error IE_Imp_AbiWord_1::importFile(const char * szFilename)
 #define TT_PAGESIZE             16      // The PageSize <pagesize> 
 #define TT_IGNOREDWORDS 17		// an ignored words section <ignoredwords>
 #define TT_IGNOREDWORD  18      // a word <iw> within an ignored words section
+#define TT_BOOKMARK     19		// <bookmark>
+#define TT_HYPERLINK	20		// <a href=>
 /*
   TODO remove tag synonyms.  We're currently accepted
   synonyms for tags, as follows:
@@ -159,8 +192,10 @@ UT_Error IE_Imp_AbiWord_1::importFile(const char * szFilename)
 ///
 static struct xmlToIdMapping s_Tokens[] =
 {
+	{	"a",			TT_HYPERLINK	},
 	{	"abiword",		TT_DOCUMENT		},
 	{	"awml",			TT_DOCUMENT		},
+	{	"bookmark",		TT_BOOKMARK		},
 	{	"br",			TT_BREAK		},
 	{	"c",			TT_INLINE		},
 	{	"cbr",			TT_COLBREAK		},
@@ -184,58 +219,73 @@ static struct xmlToIdMapping s_Tokens[] =
 
 #define TokenTableSize	((sizeof(s_Tokens)/sizeof(s_Tokens[0])))
 
-/*****************************************************************/	
-/*****************************************************************/	
-
-#define X_TestParseState(ps)	((m_parseState==(ps)))
-
-#define X_VerifyParseState(ps)	do {  if (!(X_TestParseState(ps)))			\
-									  {  m_error = UT_IE_BOGUSDOCUMENT;	\
-										 return; } } while (0)
-
-#define X_CheckDocument(b)		do {  if (!(b))								\
-									  {  m_error = UT_IE_BOGUSDOCUMENT;	\
-										 return; } } while (0)
-
-#define X_CheckError(v)			do {  if (!(v))								\
-									  {  m_error = UT_ERROR;			\
-										 return; } } while (0)
-
-#define	X_EatIfAlreadyError()	do {  if (m_error) return; } while (0)
-
-#define X_VerifyInsideBlockOrField()  do { if(!(X_TestParseState(_PS_Block) || X_TestParseState(_PS_Field))) \
-                                                                          {  m_error = UT_IE_BOGUSDOCUMENT;  \
-                                                                             return; } } while (0)
-
-/*****************************************************************/
-/*****************************************************************/
-
-void IE_Imp_AbiWord_1::_startElement(const XML_Char *name, const XML_Char **atts)
+void IE_Imp_AbiWord_1::startElement(const XML_Char *name, const XML_Char **atts)
 {
 	xxx_UT_DEBUGMSG(("startElement: %s\n", name));
 
 	X_EatIfAlreadyError();	// xml parser keeps running until buffer consumed
 	
 	UT_uint32 tokenIndex = _mapNameToToken (name, s_Tokens, TokenTableSize);
-
 	switch (tokenIndex)
 	{
 	case TT_DOCUMENT:
 		X_VerifyParseState(_PS_Init);
 		m_parseState = _PS_Doc;
+
+		{
+		  const XML_Char * lockedStyles = 0;
+		  lockedStyles = (XML_Char*)_getXMLPropValue("styles", atts);;
+		  if ((lockedStyles != NULL) && (UT_strcmp(lockedStyles, "locked") == 0))
+		    getDoc()->lockStyles(true);
+		  else
+		    getDoc()->lockStyles(false);
+		}
 		return;
 
 	case TT_SECTION:
+	{
 		X_VerifyParseState(_PS_Doc);
-		m_parseState = _PS_Sec;
-		X_CheckError(getDoc()->appendStrux(PTX_Section,atts));
-		return;
-
+		const XML_Char * pszId = (XML_Char*)_getXMLPropValue("id", atts);
+		bool bOK = true;
+		if(pszId)
+		{
+		    bOK = getDoc()->verifySectionID(pszId);
+		}
+		if(bOK)
+		{
+		    m_parseState = _PS_Sec;
+		    m_bWroteSection = true;
+		    X_CheckError(getDoc()->appendStrux(PTX_Section,atts));
+		    return;
+		}
+		else
+		{
+//
+// OK this is a header/footer with an id without a matching section. Fix this
+// now.
+//
+			const XML_Char * pszType = (XML_Char*)_getXMLPropValue("type", atts);
+			if(pszType)
+			{
+				PL_StruxDocHandle sdh = getDoc()->getLastSectionSDH();
+				getDoc()->changeSectionAttsNoUpdate(sdh,pszType,pszId);
+				m_parseState = _PS_Sec;
+				m_bWroteSection = true;
+				X_CheckError(getDoc()->appendStrux(PTX_Section,atts));
+				return;
+			}
+			m_error = UT_IE_SKIPINVALID;
+		    X_EatIfAlreadyError();
+		    return;
+		}
+	}
 	case TT_BLOCK:
 	{
 		X_VerifyParseState(_PS_Sec);
 		m_parseState = _PS_Block;
+		m_bWroteParagraph = true;
 		X_CheckError(getDoc()->appendStrux(PTX_Block,atts));
+		m_iInlineStart = getOperationCount();
 		return;
 	}
 
@@ -245,7 +295,7 @@ void IE_Imp_AbiWord_1::_startElement(const XML_Char *name, const XML_Char **atts
 		X_VerifyParseState(_PS_Block);
 		X_CheckError(_pushInlineFmt(atts));
 		X_CheckError(getDoc()->appendFmt(&m_vecInlineFmt));
-		m_iInlineStart = getOperationCount();
+		m_iInlineStart++;
 		return;
 
 		// Images and Fields are not containers.  Therefore we don't
@@ -257,7 +307,15 @@ void IE_Imp_AbiWord_1::_startElement(const XML_Char *name, const XML_Char **atts
 		X_VerifyParseState(_PS_Block);
 		X_CheckError(getDoc()->appendObject(PTO_Image,atts));
 		return;
-
+	case TT_BOOKMARK:
+		X_VerifyParseState(_PS_Block);
+		X_CheckError(getDoc()->appendObject(PTO_Bookmark,atts));
+		return;
+	case TT_HYPERLINK:
+		X_VerifyParseState(_PS_Block);
+		X_CheckError(getDoc()->appendObject(PTO_Hyperlink,atts));
+		return;
+	
 	case TT_FIELD:
 	{
 		X_VerifyParseState(_PS_Block);
@@ -422,7 +480,7 @@ void IE_Imp_AbiWord_1::_startElement(const XML_Char *name, const XML_Char **atts
 	}
 }
 
-void IE_Imp_AbiWord_1::_endElement(const XML_Char *name)
+void IE_Imp_AbiWord_1::endElement(const XML_Char *name)
 {
   	xxx_UT_DEBUGMSG(("endElement %s\n", name));
 
@@ -460,7 +518,7 @@ void IE_Imp_AbiWord_1::_endElement(const XML_Char *name)
 		X_VerifyParseState(_PS_Block);
 		X_CheckDocument(_getInlineDepth()>0);
 		// Insert a FmtMark if nothing was inserted in the block.
-		if (m_iInlineStart+1 == getOperationCount())
+		if (m_iInlineStart == getOperationCount()+1)
 		{
 			X_CheckError(getDoc()->appendFmtMark());
 		}
@@ -473,6 +531,20 @@ void IE_Imp_AbiWord_1::_endElement(const XML_Char *name)
 		X_VerifyParseState(_PS_Block);
 		return;
 
+	case TT_BOOKMARK:						// not a container, so we don't pop stack
+		UT_ASSERT(m_lenCharDataSeen==0);
+		X_VerifyParseState(_PS_Block);
+		return;
+
+	case TT_HYPERLINK:						// not a container, so we don't pop stack
+		UT_ASSERT(m_lenCharDataSeen==0);
+		X_VerifyParseState(_PS_Block);
+		// we append another Hyperlink Object, but with no attributes		
+		X_CheckError(getDoc()->appendObject(PTO_Hyperlink,NULL));
+		return;
+		
+		return;
+			
 	case TT_FIELD:						// not a container, so we don't pop stack
 		UT_ASSERT(m_lenCharDataSeen==0);
 		X_VerifyParseState(_PS_Field);
@@ -518,7 +590,7 @@ void IE_Imp_AbiWord_1::_endElement(const XML_Char *name)
 		if (trim) m_currentDataItem.del(0, trim);
 		trim = m_currentDataItem.getLength();
 		buffer = m_currentDataItem.getPointer(0);
-		while (trim >= 0 && MyIsWhite(buffer[trim])) trim--;
+		while (trim > 0 && MyIsWhite(buffer[trim])) trim--;
 		m_currentDataItem.truncate(trim+1);
 #undef MyIsWhite
  		X_CheckError(getDoc()->createDataItem((char*)m_currentDataItemName,m_currentDataItemEncoded,&m_currentDataItem,m_currentDataItemMimeType,NULL));
